//-------------------------  1994 by James G. Stout -------------------------------
// File		: cdefBar.c
// Date		: May 2, 1994
// Author	: Jim Stout
//			:
// Purpose	: A CDEF to implement a progress bar
//			:
//			: If you find a use for this, I'd love to know about it.  Bug reports
//			: are always interesting.
//			:
//			: Internet    : JimS@WRQ.COM(work hours, PST)
//			: AppleLink   : WRQ			(daily)
//			: CompuServe  : 73240,2052	(weekly or so)
//			: AOL         : JasG		(weekly or so)
//			: eWorld      : Jim Stout	(weekly or so)
//----------------------------------------------------------------------------------
//#define _DEBUGCDEF 1		// comment out to build CDEF

#include <Palettes.h>
#include <GestaltEqu.h>
#include <SetUpA4.h>

#include "colorCDEF.h"
#include "miscCDEF.h"

#include "cdefBar.h"


#ifdef _DEBUGCDEF
pascal long CDmain (short varCode, ControlHandle theControl, short message, long param);

pascal long CDmain (short varCode, ControlHandle theControl, short message, long param)
#else

//==================================================================================
// CDEF entry point
//==================================================================================
pascal long main (short varCode, ControlHandle theControl, short message, long param)
#endif
{
	cdefHdl		hCdef;
	
	RememberA0();
	SetUpA4();

	if((**theControl).contrlData)
		HLock((**theControl).contrlData);
	
	switch(message) {
		case initCntl:
			hCdef = (cdefHdl)NewHandle(sizeof(cdefData));
			if(hCdef) {
				StuffHex(&(**hCdef).thePat,"\pF0783C1E0F87C3E1");
				(**theControl).contrlData = (Handle)hCdef;
				HLock((**theControl).contrlData);
			}
		break;
		case dispCntl:
			if((**theControl).contrlData) {
				HUnlock((**theControl).contrlData);
				DisposeHandle((**theControl).contrlData);
				(**theControl).contrlData = 0;
			}
		break;
		case drawCntl:
			if((*theControl)->contrlVis) {
				if(varCode & roundBar)
					drawRoundEndBar(theControl, varCode);
				else
					drawRectBar(theControl,varCode);
			}
		break;
		case calcCRgns:
				RectRgn((RgnHandle)(param & 0x7fffffffL), &(*theControl)->contrlRect);
		break;
		case calcCntlRgn:
		case calcThumbRgn:
				RectRgn((RgnHandle)(param), &(*theControl)->contrlRect);
		break;
	}
	
	if((**theControl).contrlData)
		HUnlock((**theControl).contrlData);
		
	RestoreA4();
	return (0L);
}

//==================================================================================
// draw a simple rectangular progress bar
//==================================================================================

static void drawRectBar(ControlHandle theControl, short varCode)
{
	short			pDepth,left,bottom,inx;
	long			range,scale,filled;
	Byte			holdByte;
	Rect			r;
	Boolean			inColor=false;
	RGBColor		saveBack,saveFore;
	PenState		penSt;	
	cdefHdl			hCdef;
		
	hCdef = (cdefHdl)(**theControl).contrlData;
	if(!hCdef)
		return;
		
//----------------------------------------------------------------------------------
// Initialize for drawing
//----------------------------------------------------------------------------------
	r = (*theControl)->contrlRect;	
	pDepth = getPixDepth(&r);
	if(pDepth > 2) {
		saveColors(&saveFore, &saveBack);
		inColor = true;	
	}
		
//----------------------------------------------------------------------------------
// frame the progress bar
//----------------------------------------------------------------------------------

	frame3DRect(&r, inColor);
	InsetRect(&r,1,1);
	
	if(varCode & vertBar)
		scale = r.bottom-r.top;
	else
		scale = r.right-r.left;
	
//----------------------------------------------------------------------------------
// draw a "barber" pole status bar
//----------------------------------------------------------------------------------

	if(varCode & barberPole) {
		if(inColor) {
			setPartColor(theControl, cTingeLight, true);
			setPartColor(theControl, cFillPatColor, false);
		}
		GetPenState(&penSt);
		
#ifdef dangerousPattern
		PenPat((**hCdef).thePat);
#else
		PenPat(&(**hCdef).thePat);
#endif
		PaintRect(&r);
		SetPenState(&penSt);
		
//----------------------------------------------------------------------------------
// rotate the barber pole pattern
//----------------------------------------------------------------------------------
	
#ifdef dangerousPattern
		holdByte = (**hCdef).thePat[0];
		for(inx=0; inx<7; inx++)
			(**hCdef).thePat[inx] = (**hCdef).thePat[inx+1];
		(**hCdef).thePat[7] = holdByte;
	}
#else
		holdByte = (**hCdef).thePat.pat[0];
		for(inx=0; inx<7; inx++)
			(**hCdef).thePat.pat[inx] = (**hCdef).thePat.pat[inx+1];
		(**hCdef).thePat.pat[7] = holdByte;
	}
#endif
	
//----------------------------------------------------------------------------------
// draw a regular "thermometer style" status bar
//----------------------------------------------------------------------------------

	else {
	
//----------------------------------------------------------------------------------
// calculate "filled" portion of progress bar
//----------------------------------------------------------------------------------
	
		range = (*theControl)->contrlMax - (*theControl)->contrlMin;
		filled = (*theControl)->contrlValue * scale / range;
		
//----------------------------------------------------------------------------------
// draw the "un-filled" portion of the progress bar
//----------------------------------------------------------------------------------
	
		if(varCode & vertBar) {
			bottom = r.bottom;
			r.bottom = r.bottom-filled;						// top portion
		}
		else {
			left = r.left;
			r.left = r.left+filled;							// right portion
		}
		if(inColor) {
			setPartColor(theControl, cTingeLight, false);
		}
		EraseRect(&r);

//----------------------------------------------------------------------------------	
// draw the "filled" portion of the progress bar
//----------------------------------------------------------------------------------
		
		if(varCode & vertBar) {
			r.bottom = bottom;
			r.top = r.bottom-filled;						// bottom portion
		}
		else {
			r.left = left;
			r.right = r.left+filled;						// left portion
		}
		if(inColor) {
			setPartColor(theControl, cFillPatColor, false);
			EraseRect(&r);
		}
		else {												// in b&w, draw "filled"
			GetPenState(&penSt);							// portion in a gray pattern
			PenPat( (ConstPatternParam) "\xAA\x55\xAA\x55\xAA\x55\xAA\x55");
			PaintRect(&r);
			SetPenState(&penSt);
		}
	}
	if(inColor)
		restoreColors(&saveFore, &saveBack);	
}

//==================================================================================
// draw an round ended progress bar
//==================================================================================

static void drawRoundEndBar(ControlHandle theControl, short varCode)
{
	short			pDepth,hold,left,bottom,diameter,check,inx;
	Byte			holdByte;
	long			range,scale,filled,unfilled,min,max;
	Rect			r,rFill, rNoFill,rClip;
	Boolean			inColor=false;
	RGBColor		saveBack,saveFore;
	PenState		penSt;
	RgnHandle		c;	
	cdefHdl			hCdef;
		
	hCdef = (cdefHdl)(**theControl).contrlData;
	if(!hCdef)
		return;

//----------------------------------------------------------------------------------
// Initialize for drawing
//----------------------------------------------------------------------------------
	rFill = (*theControl)->contrlRect;	
	pDepth = getPixDepth(&rFill);
	if(pDepth > 2) {
		saveColors(&saveFore, &saveBack);
		inColor = true;	
	}

	c = NewRgn();
	GetClip(c);
	
//----------------------------------------------------------------------------------
// draw the frame of the progress bar - 3D look
//----------------------------------------------------------------------------------

	if(varCode & vertBar)
		diameter = rFill.right - rFill.left;
	else
		diameter = rFill.bottom - rFill.top;

	frame3DRoundEndBar(&rFill, diameter, inColor);
	InsetRect(&rFill, diameter/5, diameter/5);
	r = rNoFill = rFill;
	
//----------------------------------------------------------------------------------
// draw a "barber" pole status bar
//----------------------------------------------------------------------------------

	if(varCode & barberPole) {
		if(inColor) {
			setPartColor(theControl, cTingeLight, true);
			setPartColor(theControl, cFillPatColor, false);
		}
		GetPenState(&penSt);
		
#ifdef dangerousPattern
		PenPat((**hCdef).thePat);
#else
		PenPat(&(**hCdef).thePat);
#endif
		PaintRoundRect(&rFill, diameter, diameter);
		SetPenState(&penSt);
		
//----------------------------------------------------------------------------------
// rotate the barber pole pattern
//----------------------------------------------------------------------------------
	
#ifdef dangerousPattern
		holdByte = (**hCdef).thePat[0];
		for(inx=0; inx<7; inx++)
			(**hCdef).thePat[inx] = (**hCdef).thePat[inx+1];
		(**hCdef).thePat[7] = holdByte;
#else
		holdByte = (**hCdef).thePat.pat[0];
		for(inx=0; inx<7; inx++)
			(**hCdef).thePat.pat[inx] = (**hCdef).thePat.pat[inx+1];
		(**hCdef).thePat.pat[7] = holdByte;
#endif
	}
	
//----------------------------------------------------------------------------------
// draw a regular "thermometer style" status bar
//----------------------------------------------------------------------------------

	else {
	
//----------------------------------------------------------------------------------
// calculate "filled" portion of progress bar
//----------------------------------------------------------------------------------

		if(varCode & vertBar) {
			diameter = rFill.right - rFill.left;
			scale = rFill.bottom-rFill.top;
		}
		else {
			diameter = rFill.bottom - rFill.top;
			scale = rFill.right-rFill.left;
		}
	
		range = (*theControl)->contrlMax - (*theControl)->contrlMin;
		filled = (*theControl)->contrlValue * scale / range;
		max = (*theControl)->contrlMax * scale / range;
		unfilled = max-filled;
		
//----------------------------------------------------------------------------------
// set up the rects for the "filled" and "unfilled" areas of the bar
//----------------------------------------------------------------------------------
	
		if(varCode & vertBar) {
			if(unfilled < diameter) {
				rNoFill.bottom = rNoFill.top+diameter+diameter/2;
			}
			else {
				rNoFill.bottom = rNoFill.top+unfilled+diameter/2;
				if(filled < diameter/2)
					rNoFill.bottom = r.bottom;
			}
			
			if(filled < diameter) {
				rFill.top = rFill.bottom-diameter-diameter/2;
			}
			else {
				rFill.top = rFill.bottom-filled-diameter/2;
				if(unfilled < diameter/2)
					rFill.top = r.top;
			}
			rClip = rNoFill;
			rClip.bottom = rClip.top+unfilled;
		}
		else {
			if(unfilled < diameter) {
				rNoFill.left = rNoFill.right-diameter-diameter/2;
			}
			else {
				rNoFill.left = rNoFill.right-unfilled-diameter/2;
				if(filled < diameter/2)
					rNoFill.left = r.left;
			}
			
			if(filled < diameter) {
				rFill.right = rFill.left+diameter+diameter/2;
			}
			else {
				rFill.right = rFill.left+filled+diameter/2;
				if(unfilled < diameter/2)
					rFill.right = r.right;
			}
			rClip = rNoFill;
			rClip.left = rClip.right-unfilled;
		}
		
//----------------------------------------------------------------------------------
// draw the "unfilled" portion of the progress bar
//----------------------------------------------------------------------------------
	
		if(inColor) {
			setPartColor(theControl, cTingeLight, false); 
		}
		ClipRect(&rClip);
		EraseRoundRect(&rNoFill, diameter, diameter);
	
//----------------------------------------------------------------------------------	
// draw the "filled" portion of the progress bar
//----------------------------------------------------------------------------------
	
		rClip = rFill;
		if(varCode & vertBar)
			rClip.top = rClip.bottom-filled;
		else
			rClip.right = rClip.left+filled;
			
		ClipRect(&rClip);
		
		if(inColor) {
			setPartColor(theControl, cFillPatColor, false);
			EraseRoundRect(&rFill, diameter, diameter);
		}
		else {												// in b&w, draw "filled"
			GetPenState(&penSt);							// portion in a gray pattern
			PenPat( (ConstPatternParam) "\xAA\x55\xAA\x55\xAA\x55\xAA\x55");
			PaintRoundRect(&rFill, diameter, diameter);
			SetPenState(&penSt);
			FrameRoundRect(&rFill, diameter, diameter);
		}

//----------------------------------------------------------------------------------		
// make it look rounded
//----------------------------------------------------------------------------------
	
		if(inColor) {
			setPartColor(theControl, cTingeLight, true);
			if(varCode & vertBar) {
				MoveTo(r.left+diameter/4, r.bottom-diameter/2);
				LineTo(r.left+diameter/4, r.top+diameter/2);
			}
			else {
				MoveTo(r.left+diameter/2, r.top+diameter/4);
				LineTo(r.right-diameter/2, r.top+diameter/4);
			}
		}
	}
	SetClip(c);
	DisposeRgn(c);
	
	if(inColor)
		restoreColors(&saveFore, &saveBack);
	else
		ForeColor(blackColor);
}

//==================================================================================
// frame a rect with an inset 3D effect by drawing right & bottom edges in white
// and left & top in black.
//==================================================================================
static void frame3DRect(Rect *rr, Boolean inColor)
{
	Rect		r;
	RGBColor	rgbBack;
	
	if(inColor) {
		GetBackColor(&rgbBack);
		if(rgbBack.red == 65535 &&
			rgbBack.green == 65535 &&
			rgbBack.blue == 65535)
			inColor = false;
	}
	r = *rr;
	if(inColor)
		ForeColor(whiteColor);
	MoveTo(r.right-1, r.top+1);
	LineTo(r.right-1, r.bottom-1);
	LineTo(r.left,  r.bottom-1);
	ForeColor(blackColor);
	MoveTo(r.left,  r.bottom-2);
	LineTo(r.left,  r.top);
	LineTo(r.right-1, r.top);
}

//==================================================================================
// draw a bar with rounded ends, with a 3D effect by drawing the frame in white
// from 45-225 & black from 225-45 degrees.
//==================================================================================
static void frame3DRoundEndBar(Rect *rr, short diameter, Boolean inColor)
{
	Rect		r, r2;
	Boolean		vert=false;
	RGBColor	rgbBack;
	
	if(inColor) {
		GetBackColor(&rgbBack);
		if(rgbBack.red == 65535 &&
			rgbBack.green == 65535 &&
			rgbBack.blue == 65535)
			inColor = false;
	}
	r = r2 = *rr;
	
	if((r.right-r.left) < (r.bottom-r.top))
		vert = true;

	if(vert) {
		r2.bottom = r2.top + diameter;
		if(inColor)
			ForeColor(whiteColor);
		FrameArc(&r2, 45, 45);
		MoveTo(r.right-1, r.top+diameter/2);
		LineTo(r.right-1, r.bottom-diameter/2);
		r2.bottom = r.bottom;
		r2.top = r.bottom-diameter;
		FrameArc(&r2, 90, 135);

		ForeColor(blackColor);
		FrameArc(&r2, 225, 45);
		MoveTo(r.left, r.bottom-diameter/2);
		LineTo(r.left, r.top+diameter/2);
		r2.top = r.top;
		r2.bottom = r2.top+diameter;
		FrameArc(&r2, 270, 135); 
	}
	else {
		if(inColor)
			ForeColor(whiteColor);
		r2.left = r2.right - diameter;
		FrameArc(&r2, 45, 135);
		MoveTo(r.right-diameter/2, r.bottom-1);
		LineTo(r.left+diameter/2, r.bottom-1);
		r2.left=r.left;
		r2.right=r.left+diameter;
		FrameArc(&r2, 180, 45);
		
		ForeColor(blackColor);
		FrameArc(&r2, 225, 135);
		MoveTo(r.left+diameter/2, r.top);
		LineTo(r.right-diameter/2, r.top);
		r2.right=r.right;
		r2.left=r.right-diameter;
		FrameArc(&r2, 0, 45);
	}
}
