//-----------------------  1992-1994 by James G. Stout ----------------------------
// File		: cdefDtc.c
// Date		: 20 June 1992
// Author	: Jim Stout
//			:
// Purpose	: a "Date Time Control" CDEF that allows adjustment of date
//			: and time as in the General Control Panel. Clicking on the
//			: date or time will show a 'small arrow' type control for
//			: date or time adjustment.
//			:
//			: see cdefDtc.h for more detail and variation codes
//			:
//			: If you find a use for this, I'd love to know about it.  Bug reports
//			: are always interesting.
//			:
//			: Internet    : JimS@WRQ.COM(work hours, PST)
//			: AppleLink   : WRQ			(daily)
//			: CompuServe  : 73240,2052	(weekly or so)
//			: AOL         : JasG		(weekly or so)
//			: eWorld      : Jim Stout	(weekly or so)
//----------------------------------------------------------------------------------
//#define _DEBUGCDEF 1

#include <QDOffscreen.h>
#include <GestaltEqu.h>
#include <LoMem.h>
#include <Packages.h>
#include <SetUpA4.h>

#include "colorCDEF.h"
#include "grayCDEF.h"
#include "miscCDEF.h"
#include "qdCDEF.h"

#include "cdefDtc.h"

#ifdef _DEBUGCDEF
pascal long CDmain (short, ControlHandle, short, long);

pascal long CDmain (short varCode, ControlHandle theControl, short message, long param)
#else

//==================================================================================
// CDEF entry point
//==================================================================================
pascal long main (short varCode, ControlHandle theControl, short message, long param)
#endif
{
	long		ret = 0L;
	short		txF,txS,partCode,myVarCode=0;
	GrafPtr		thisPort;
		
	RememberA0();
	SetUpA4();
	
	HLock((Handle)theControl);
	
//----------------------------------------------------------------------------------
// make sure we've got the correct font
//----------------------------------------------------------------------------------	

	if(!(varCode & useWindFont)) {			// don't use the window font	
		GetPort(&thisPort);
		txF = thisPort->txFont;
		txS = thisPort->txSize;
		thisPort->txFont = SysFontFam;		// use the System font & size	
		thisPort->txSize = SysFontSize;
	}

//----------------------------------------------------------------------------------
// lock down the cdef data and get our 'custom' variation code
//----------------------------------------------------------------------------------	

	if((**theControl).contrlData) {
		HLock((**theControl).contrlData);
		myVarCode = (**(dtcHandle)(**theControl).contrlData).varCode;
	}
	partCode = LoWord(param);
	
//----------------------------------------------------------------------------------
// handle the standard control messages	
//----------------------------------------------------------------------------------
	
	switch(message) {
		case initCntl:
			doInit(theControl, varCode, param);
		break;
		case dispCntl:
			doDisp(theControl, varCode, param);
		break;
		case testCntl:
			ret = doTest (theControl, myVarCode, param);
		break;
		case drawCntl:
			if((**theControl).contrlVis != 0 &&
				((WindowPeek)(**theControl).contrlOwner)->visible) {
				doDraw(theControl, myVarCode, partCode);
			}
		break;
		case calcCRgns:
			RectRgn((RgnHandle)(param & 0x7fffffffL), &(**theControl).contrlRect);
		break;
		case calcCntlRgn:
		case calcThumbRgn:
			RectRgn((RgnHandle)(param), &(**theControl).contrlRect);
		break;
	}
	
//----------------------------------------------------------------------------------
// restore window font & size info	
//----------------------------------------------------------------------------------

	if(!(varCode & useWindFont)) {
		thisPort->txFont = txF;
		thisPort->txSize = txS;
	}

	if((**theControl).contrlData)
		HUnlock((**theControl).contrlData);
	HUnlock((Handle)theControl);
	
	RestoreA4();
	return (ret);
}
 
//==================================================================================
//	initialize our data
//==================================================================================

static long doInit		(ControlHandle theControl, short varCode, long param)
{
	dtcHandle	hDtc;
	Intl0Hndl	hInt;
	short		inx;

	hDtc = (dtcHandle) NewHandle(sizeof(dtcData));
	if(hDtc) {
		(**theControl).contrlData = (Handle)hDtc;
		for(inx=0;inx<6;inx++)
			(**hDtc).poly[inx] = 0;
		(**hDtc).arrowHilite = 0;
		(**hDtc).varCode = varCode;					// save original varCode			
		(**hDtc).qdVers = getQDVers();				// QuickDraw version				
		(**hDtc).offPort = 0;						// gets created in doDraw			
		(**hDtc).pDepth = 1;			
		(**hDtc).currSel = 0;
		
		hInt = (Intl0Hndl)IUGetIntl(0);
		
		(**hDtc).dateSep = (**hInt).dateSep;
		(**hDtc).dateOrder = (**hInt).dateOrder;
		(**hDtc).timeSep = (**hInt).timeSep;
		(**hDtc).timeCycle = (**hInt).timeCycle;
		
//----------------------------------------------------------------------------------
// contrlMax & controlMin are used as "extended" variation codes
//----------------------------------------------------------------------------------

		if(!(**theControl).contrlMax) {				// use ControlPanel 12/24 setting
			if((**hInt).timeCycle != 0)				// set for AM/PM
				(**hDtc).varCode ^= showAMPM;		// so, set our varCode		
		}
		else										// ignore and leave at 24 hour
			(**theControl).contrlMax = 0;
			
		if((**theControl).contrlMin) {				// if non-zero, wants 3D arrows
			(**hDtc).varCode ^= ctl3D;				// so, set our varCode
			(**theControl).contrlMin = 0;
		}
		HLock((Handle)hDtc);
	}
	else
		(**theControl).contrlData = 0;
}

//==================================================================================
//	dispose of our data
//==================================================================================

static long doDisp		(ControlHandle theControl, short varCode, long param)
{
	dtcHandle	hDtc;
	short		inx;
	BitMap		*theBits;
	
	hDtc = (dtcHandle)(**theControl).contrlData;
	if(hDtc) {
		if((**hDtc).qdVers < gestalt32BitQD12) {				// offport is bitmap	
			theBits = &((GrafPtr)(**hDtc).offPort)->portBits;
			DisposePtr((*theBits).baseAddr);
			ClosePort((GrafPtr)(**hDtc).offPort);
			DisposePtr((Ptr)(**hDtc).offPort);
		}
		else
			DisposeGWorld((**hDtc).offPort);					// offPort is GWorld	
				
		for(inx=0;inx<6;inx++) {
			if((**hDtc).poly[inx])
				KillPoly((**hDtc).poly[inx]);
		}
		HUnlock((Handle)hDtc);
		DisposeHandle((Handle)hDtc);
		(**theControl).contrlData = 0;
	}
}

//==================================================================================
//	check for a mouseDown in our control	
//==================================================================================

static long doTest (ControlHandle theControl, short varCode, long param)
{
	Point		p;
	short		inx,start, end;
	long		ret=0L;
	Rect		parts[NUMPARTS];
	Boolean 	hit;
	dtcHandle	hDtc;
	
	hDtc = (dtcHandle)(**theControl).contrlData;
	if(!hDtc)
		return(ret);
	
	p.h = LoWord(param);									// get the mouse hit point		
	p.v = HiWord(param);
	
	start = HOURS;											// set loop variables			
	end = DOWNDATE;

	if(varCode & timeOnly)
		end = DOWNTIME;
	else
	if(varCode & dateOnly)
		start = DT1;

	if(PtInRect(p,&(**theControl).contrlRect)) {			// click in control				
		getRects(theControl, varCode, parts);
		(**hDtc).arrowHilite = 0;							// arrow not hilited
		
//----------------------------------------------------------------------------------
// first, test for a simple click in one of the digit fields
//----------------------------------------------------------------------------------

		for(inx = start;inx <= end;inx++) {					// loop through all parts		
			if(PtInRect(p,&parts[inx])) {					// and check for a mouse down
				if(!arrowPart(inx))	{						// click in a digit, not in
					(**theControl).contrlValue = inx;		// arrow.
					return((long)inx);						// all done
				}
			}
		}

//----------------------------------------------------------------------------------
// second, check for a click in an arrow
//----------------------------------------------------------------------------------
					
		if((**theControl).contrlValue) {					// an arrow is visible
			if((**theControl).contrlValue <= AMPM) {		// time arrow
				start = UPTIME;
				end = DOWNTIME;
			}
			else {											// date arrow
				start = UPDATE;
				end = DOWNDATE;
			}
			for (inx=start;inx<=end;inx++) {				// check for press in arrow
				if(PtInRect(p,&parts[inx])) {				
					while (StillDown()) {					// and track it		
						if(PtInRect(p,&parts[inx]))	{		// mouse hasn't moved so	
							(**hDtc).arrowHilite = inx;		// hilite & update value
							updateValue (theControl, varCode, inx);
							hit = TRUE;
							ret = (long)inx;
						}
						else												
						if(hit) {							// mouse moved outside arrow	
							(**hDtc).arrowHilite = 0;		// so un-hilite arrow
							doDraw(theControl, varCode, ALL);
							hit = FALSE;
							ret = 0L;
						}
						GetMouse(&p);						// get new mouse location	
					}
				}
			}
			if(hit) {										// un-hilite arrow, mouse
				(**hDtc).arrowHilite = 0;					// came up...
				doDraw(theControl, varCode, ALL);
				ret = 0;
			}
		}
	}
	return(ret);
}

//==================================================================================
//	update the control value	
//==================================================================================

static void updateValue	(ControlHandle theControl, short varCode, short partCode)
{
	short			selected,m,d,y,dateTbl[3];
	unsigned long	secs;
	DateTimeRec		dtr;
	dtcHandle	hDtc;
	
	hDtc = (dtcHandle)(**theControl).contrlData;
	if(!hDtc)
		return;
	
	if(!arrowPart(partCode))				// only track the up/down arrows			
		return;
	
	selected = (**theControl).contrlValue;	// the item to adjust						
	secs = GetCRefCon(theControl);			// get current date/time setting			
	Secs2Date(secs,&dtr);					// convert to a DateTimeRecord				
	
	getDateFmt(theControl, &m, &d, &y);
	dateTbl[d] = dtr.day;
	dateTbl[m] = dtr.month;
	dateTbl[y] = dtr.year;
	
	switch(selected) {						// now, bump the selected item up or down	
		case MINUTES:
			if(partCode == UPTIME)
				secs += 60;
			else
				secs -= 60;
		break;
		case HOURS:
			if(partCode == UPTIME)
				secs += 3600;
			else
				secs -= 3600;
		break;
		case AMPM:
			if(dtr.hour < 12)
				dtr.hour+=12;
			else
				dtr.hour-=12;
		break;
		case DT1:
			if(partCode == UPDATE)
				dateTbl[m]++;
			else
				dateTbl[m]--;
			if(dateTbl[m] > 12) {
				dateTbl[y]++;
				dateTbl[m] = 1;
			}
			else
			if(dateTbl[m] < 1) {
				dateTbl[y]--;
				dateTbl[m] = 12;
			}
		break;
		case DT2:
			if(partCode == UPDATE)
				dateTbl[d]++;
			else
				dateTbl[d]--;
		break;
		case DT3:
			if(partCode == UPDATE)
				dateTbl[y]++;
			else
			if(dateTbl[y] > 1904)				// ROM routine only good since 1904,	
				dateTbl[y]--;					// the year the Mac was invented !		
		break;
	}
	dtr.day = dateTbl[d];
	dtr.month = dateTbl[m];
	dtr.year = dateTbl[y];
	
	if(selected != HOURS && selected != MINUTES)
		Date2Secs(&dtr, &secs);					// convert date rec back to seconds		
		
	SetCRefCon(theControl,secs);				// save the new date & time	
	doDraw(theControl, varCode, ALL);			// draw the new date & time	
	Delay(10L, (long *)&secs);					// pause a bit 					
}

//==================================================================================
//	process the drawCntl message	
//==================================================================================
 
void doDraw(ControlHandle theControl, short varCode, short partCode)
{
	WindowPeek			w;
	RgnHandle			c;
	Boolean				inactive=false,inColor=false,bgInColor=false;
	Boolean				haveGW=false,haveGray=false;
	PenState			penSt;
	unsigned long		secs;
	short				h,v,oldVal,dtLeft,toDraw;
	Str31				s1;
	Rect				r,bitRect,arrowRect;
	BitMap				*offBits;
	Rect				parts[NUMPARTS];
	CGrafPtr			cdefPort;
	GDHandle			cdefgDev;
	PixMapHandle		pmHdl;
	RGBColor			saveFore,saveBack;
	RGBColor			rgbBlack = {0,0,0};
	RGBColor			rgbWhite = {-1,-1,-1};
	dtcHandle			hDtc;
	
	hDtc = (dtcHandle)(**theControl).contrlData;
	if(!hDtc)
		return;
		
	w = (WindowPeek)(**theControl).contrlOwner;				// don't draw if window is	
	if(!w->visible || arrowPart(partCode))	{				// invisible or if partCode	
		return;												// is an up/down arrow		
	}			
	if((**theControl).contrlHilite == 0xFF) {
		inactive = true;
	}
	else							
	if((**theControl).contrlHilite > 0 &&					// we hilite differently 	
		!arrowPart((short)(**theControl).contrlHilite)) {	// from normal controls		
		return;											
	}
	
	getRects(theControl, varCode, parts);					// rects for control parts	
	
//----------------------------------------------------------------------------------
// now, update our GWorld if required, and set it as our drawing port	
//----------------------------------------------------------------------------------

	(**hDtc).pDepth = getOff(&(**hDtc).offPort, &parts[ALL]);
	
	if((**hDtc).pDepth == 0) {								// oh oh				
		return;				
	}
	
	if((**hDtc).qdVers >= gestalt32BitQD12) {				// have a GWorld
		haveGW = true;
		pmHdl = getLockedPixels(&(**hDtc).offPort, (**hDtc).qdVers);
		if(!pmHdl)
			return;											// nuts
	}
	
//----------------------------------------------------------------------------------
// initialize our drawing stuff	
//----------------------------------------------------------------------------------

	r = (**theControl).contrlRect;							// the rect we draw in		
	c = NewRgn();	
	GetClip(c);												// get clipping region
	ClipRect(&r);											// clip to our rect	

	if((**hDtc).pDepth > 2) {								// save current port
		inColor = true;										// colors
		saveColors(&saveFore, &saveBack);
		bgInColor = true;
		if(saveBack.red == 65535 &&							// is bg white?
			saveBack.green == 65535 &&
			saveBack.blue == 65535)
			bgInColor = false;
	}	
#ifdef NOEMBOSS
	bgInColor = false;
#endif	
//----------------------------------------------------------------------------------
// Set for drawing in offscreen port
//----------------------------------------------------------------------------------
	if(haveGW) {											// set offscreen for drawing
		GetGWorld(&cdefPort, &cdefgDev);	
		SetGWorld((**hDtc).offPort, nil);
	} else {
		GetPort((GrafPtr*)&cdefPort);
		SetPort((GrafPtr)(**hDtc).offPort);
	}
	if(inColor) {											// set colors in offPort	
		RGBForeColor(&saveFore);
		RGBBackColor(&saveBack);
		setPartColor(theControl, cTextColor, true);
	}
	else {
		ForeColor(blackColor);
		BackColor(whiteColor);
	}
	if(varCode & useWindFont) {								// set font in offPort		
		(**hDtc).offPort->txFont = cdefPort->txFont;
		(**hDtc).offPort->txSize = cdefPort->txSize;
	}
	
	if(haveGW) {											// we have a pixMap	
		offBits = (BitMap *)*pmHdl;
		EraseRect(&offBits->bounds);
	}
	else {													// we have a bitMap			
		offBits = (BitMap *)&((GrafPtr)(**hDtc).offPort)->portBits;
	}
			
//----------------------------------------------------------------------------------
// Draw the control title
//----------------------------------------------------------------------------------

	OffsetRect(&parts[TITLE], -parts[ALL].left, -parts[ALL].top);
	h = parts[TITLE].left + 1;
	v = parts[TITLE].bottom-parts[FONTINFO].bottom;
	if(bgInColor && varCode & ctl3D) {
		RGBForeColor(&rgbWhite);
		MoveTo(h+1,v+1);
		DrawString((**theControl).contrlTitle);
		setPartColor(theControl, cTextColor, true);
	}
	if(inactive && inColor) {								// control is inactive		
		if(haveGrayText()) {								// draw in gray
			TextMode(grayishTextOr);
			haveGray = true;
		}
	}
	MoveTo(h,v);
	DrawString((**theControl).contrlTitle);					// draw the title
	TextMode(srcOr);
	
//----------------------------------------------------------------------------------
// Get the refCon - our control 'value' or initialize if needed
//----------------------------------------------------------------------------------

	secs = GetCRefCon(theControl);							// control 'value' in refCon	
	if(secs == 0) {											// nothing there
		GetDateTime(&secs);						
		SetCRefCon(theControl, secs);						// plug in current time
	}

//----------------------------------------------------------------------------------
// Convert to a date string and draw date
//----------------------------------------------------------------------------------
	
	if(!(varCode & timeOnly)) {
		getDateStr(theControl, secs, s1);
		OffsetRect(&parts[DATE], -parts[ALL].left, -parts[ALL].top);
		h = parts[DATE].left + 1;
		v = parts[DATE].bottom-parts[FONTINFO].bottom;
		if(bgInColor && varCode & ctl3D) {					// embossed 3D effect
			RGBForeColor(&rgbWhite);
			MoveTo(h+1,v+1);
			DrawString(s1);
			setPartColor(theControl, cTextColor, true);
		}
		if(haveGray)										// means inactive
			TextMode(grayishTextOr);
		MoveTo(h,v);
		DrawString(s1);
		TextMode(srcOr);
	}
	
//----------------------------------------------------------------------------------
// Convert to a time string and draw time
//----------------------------------------------------------------------------------

	if(!(varCode & dateOnly)) {
		getTimeStr(theControl, varCode, secs, s1);
		OffsetRect(&parts[HOURS], -parts[ALL].left, -parts[ALL].top);
		h = parts[HOURS].left + 1;
		v = parts[HOURS].bottom-parts[FONTINFO].bottom;
		if(bgInColor && varCode & ctl3D) {					// embossed 3D effect
			RGBForeColor(&rgbWhite);
			MoveTo(h+1,v+1);
			DrawString(s1);
			setPartColor(theControl, cTextColor, true);
		}
		if(haveGray)										// means inactive
			TextMode(grayishTextOr);
		MoveTo(h,v);
		DrawString(s1);
		TextMode(srcOr);
	}
	TextMode(srcOr);

	if(partCode == ALL)
		oldVal = (**theControl).contrlValue;
	else
	
//----------------------------------------------------------------------------------
// set new value for control - actually, this indicates the hilited digits, 
// the actual dateTime value is stored in the control refcon!
//----------------------------------------------------------------------------------	

	if(!arrowPart(partCode)) {

//----------------------------------------------------------------------------------
// if inactive control, reset control value to deselect digits
//----------------------------------------------------------------------------------

		if((**theControl).contrlHilite == 0xFF) {	
			(**hDtc).currSel = (**theControl).contrlValue = 0;	
		}
		else {
		
//----------------------------------------------------------------------------------
// if click on currently selected digits, deselect & remove hiliting
// by setting value to 0
//----------------------------------------------------------------------------------

			if((**theControl).contrlValue == partCode) {
				if((**hDtc).currSel == (**theControl).contrlValue) {		
					(**hDtc).currSel = (**theControl).contrlValue = 0;
				}
				else
					(**hDtc).currSel = (**theControl).contrlValue;
			}

//----------------------------------------------------------------------------------
// click on new digits, select by setting value to hilite
//----------------------------------------------------------------------------------

			else {
				if(partCode < 0xFF) {
					(**theControl).contrlValue = partCode;
					(**hDtc).currSel = 0;	
				}
			}
		}
	}

//----------------------------------------------------------------------------------
// 	Draw the arrows into the offscreen port if needed
//----------------------------------------------------------------------------------
	if((**theControl).contrlValue && !inactive) {			// only if selected	& active
		
		if(!(**hDtc).poly[0])								// create polygons for
			makePolys(theControl, varCode);					// arrows
			
		if((**hDtc).arrowHilite)							// arrow to draw hilited
			toDraw = (**hDtc).arrowHilite;
		else
		if(partCode)										
			toDraw = partCode;								// arrow not hilited
		else
			toDraw = (**theControl).contrlValue;			// arrow not hilited
	
		drawArrows(theControl, varCode, toDraw, inColor);
	}

//----------------------------------------------------------------------------------
// use CopyBits to draw the control onscreen
//----------------------------------------------------------------------------------

	if(haveGW) {											// back to onscreen port	
		SetGWorld(cdefPort, cdefgDev);
	} else {
		SetPort((GrafPtr)cdefPort);
	}
	if(inColor) {
		RGBForeColor(&rgbBlack);
		RGBBackColor(&rgbWhite);
	}
	CopyBits(offBits,										// from bitmap				
			&((GrafPtr)cdefPort)->portBits, 				// to onscreen port			
			&(**hDtc).offPort->portRect,					// from rect				
			&parts[ALL], 									// to rect					
			srcCopy, nil);
	
	if((**hDtc).qdVers >= gestalt32BitQD13)
		UnlockPixels(pmHdl);
	if(inColor) {
		restoreColors(&saveFore, &saveBack);
	}

//----------------------------------------------------------------------------------
// hilite the selected digits - contrlValue is currently selected field 
//----------------------------------------------------------------------------------

	if(partCode == ALL)	{									// entire control redrawn	
		if(oldVal)
			doHilite(theControl, varCode, oldVal);
	}
	else {
		doHilite(theControl, varCode, 						// hilite part clicked on
			(**theControl).contrlValue);		
	}

//----------------------------------------------------------------------------------
// if inactive & no GrayishTextOr, gray out the control the old way	
//----------------------------------------------------------------------------------

	if(inactive && !haveGray) {
		GetPenState(&penSt);	
		PenPat( (ConstPatternParam) "\xAA\x55\xAA\x55\xAA\x55\xAA\x55");
		PenMode(patBic);
		PaintRect(&r);
		SetPenState(&penSt);
	}
	SetClip(c);												// restore clip & pen	
	DisposeRgn(c);
}

//----------------------------------------------------------------------------------
//	Hilite the digit parts of the control
//----------------------------------------------------------------------------------

static void doHilite(ControlHandle theControl, short varCode, short partCode)
{	
	Rect	parts[NUMPARTS];
	dtcHandle	hDtc;
	
	hDtc = (dtcHandle)(**theControl).contrlData;
	if(!hDtc)
		return;
	
	if ((partCode >= HOURS && partCode <= AMPM) ||
		(partCode >= DT1 && partCode <= DT3)) {
		
		getRects(theControl, varCode, parts);
		BitClr(&HiliteMode,pHiliteBit);
		InvertRect(&parts[partCode]);
		ValidRect(&parts[partCode]);
	}
}
//==================================================================================
//	Draw the actual arrow control image.
//	
//==================================================================================

static void drawArrows(ControlHandle theControl, short varCode, short toDraw, 
						Boolean inColor)
{
	short		inx,base;
	Rect		arrowRect,parts[NUMPARTS];
	RGBColor	rgbBlack = {0,0,0};
	RGBColor	rgbWhite = {-1,-1,-1};
	RGBColor	inGray = {61166, 61166, 61166};
	RGBColor	bgGray = {56797, 56797, 56797};
	RGBColor	fgGray = {30583, 30583, 30583};
	RGBColor	grayColor;
	dtcHandle	hDtc;
	
	hDtc = (dtcHandle)(**theControl).contrlData;
	
	getRects(theControl, varCode, parts);				// rects for control parts
	
	if(toDraw < DT1)
		base = 0;
	else
		base = 1;
//----------------------------------------------------------------------------------
// Frame the arrow box then fill & shadow if 3D
//----------------------------------------------------------------------------------

	if((**hDtc).poly[base*3]) {
		if(inColor) {
			if(varCode & ctl3D) {					// 3D filled in gray
				RGBForeColor(&bgGray);
				PaintPoly((**hDtc).poly[base*3]);	// fill the box
				RGBForeColor(&rgbBlack);
			}
			else									// use cctb frame color
				setPartColor(theControl, cFrameColor, true);
		}
		FramePoly((**hDtc).poly[base*3]);
		
//----------------------------------------------------------------------------------
// draw shadow if 3D arrow
//----------------------------------------------------------------------------------

		if(inColor && varCode & ctl3D) {
			if(toDraw < DT1) {
				arrowRect = parts[FRAMET];
				OffsetRect(&arrowRect, -parts[ALL].left, -parts[ALL].top);
			}
			else {
				arrowRect = parts[FRAMED];
				OffsetRect(&arrowRect, -parts[ALL].left, -parts[ALL].top);
			}
			RGBForeColor(&fgGray);
			MoveTo(arrowRect.left+2, arrowRect.bottom-2);
			LineTo(arrowRect.right-3, arrowRect.bottom-2);
			LineTo(arrowRect.right-3, arrowRect.bottom-3);
			LineTo(arrowRect.right-2, arrowRect.bottom-3);
			LineTo(arrowRect.right-2, arrowRect.top+2);
			setPartColor(theControl, cFrameColor, true);
		}
	}
	
//----------------------------------------------------------------------------------
// The small, non-3D arrows are hilited by filling the top/bottom half
// of the frame with the bgColor.  Then we fill/paint the arrow poly in white.
//----------------------------------------------------------------------------------

	if(!(varCode & ctl3D) || !inColor) {
		if((**hDtc).arrowHilite) {
			arrowRect = parts[toDraw];
			OffsetRect(&arrowRect, -parts[ALL].left, -parts[ALL].top);
			if(toDraw == UPDATE || toDraw == UPTIME)
				MoveTo(arrowRect.left+1, arrowRect.top+1);
			else
				MoveTo(arrowRect.left+1,arrowRect.top-1);
			for(inx=0;inx<8;inx++) {
				Line(8,0);
				Move(-8,1);
			}
		}
	}
//----------------------------------------------------------------------------------	
// draw both the top & bottom arrows (poly[1] & poly[2])
//----------------------------------------------------------------------------------

	for(inx=1;inx<3;inx++) {
		if((**hDtc).poly[base*3+inx]) {					// Need the poly!
//----------------------------------------------------------------------------------
// First check to see if we are drawing a hilited arrow
//----------------------------------------------------------------------------------
			if(toDraw-4 == inx ||
				toDraw-9 == inx) {						// arrow is hilited
				if(inColor) {
					if(varCode & ctl3D)
						RGBForeColor(&rgbBlack);		// in black for 3D arrows
					else
						RGBForeColor(&rgbWhite);		// in white for non-3D
				}
				else									// b&w hilited arrow
					ForeColor(whiteColor);				// must draw white				
				PaintPoly((**hDtc).poly[base*3+inx]);	// draw the arrow
			}
//----------------------------------------------------------------------------------
// Nope, it is not hilited
//----------------------------------------------------------------------------------
			else											// make it look like a
			if(varCode & ctl3D) {							// scroll bar arrow
				if(inColor) {								// with lighter center
					setPartColor(theControl, cTingeLight, true);
					GetForeColor(&grayColor);
					grayColor.red -= 13107;
					grayColor.green -= 13107;
					grayColor.blue -= 13107;
					RGBForeColor(&grayColor);
					PaintPoly((**hDtc).poly[base*3+inx]);
					setPartColor(theControl, cTingeDark, true);
				}
				else										// in 3D, but in b&w
					PaintPoly((**hDtc).poly[base*3+inx]);	// so arrow is solid fgColor
			}
			else											// not 3D, so arrow
				PaintPoly((**hDtc).poly[base*3+inx]);		// is solid fgColor
				
			FramePoly((**hDtc).poly[base*3+inx]);			// frame the arrow
		}
	
		if(inColor)											// set fg for next arrow
			setPartColor(theControl, cFrameColor, true);
		else
			ForeColor(blackColor);
	}
}

//==================================================================================
//	Get the rectangles for each part of the control.  These are used for
//	detecting mouse actions and hiliting the date & time digits.
//==================================================================================
static void getRects (ControlHandle theControl, short varCode, Rect parts[])
{
	short		ht,wid,tWid,offset,inx,m,d,y;
	Rect 		base,work,up,down,mR,dR,yR;
	FontInfo	f;
	char		sStr[9];
	dtcHandle	hDtc;
	
	hDtc = (dtcHandle)(**theControl).contrlData;
	if(!hDtc)
		return;
	
	sStr[0] = 8;									// sample "88/88/88"		
	sStr[3] = sStr[6] = (**hDtc).dateSep;
	sStr[1] = sStr[2] = sStr[4] = sStr[5] = sStr[7] = sStr[8] = '8'; 
	
	getDateFmt(theControl, &m, &d, &y);
	GetFontInfo(&f);
	ht = f.ascent + f.descent+f.leading;
	if(f.leading == 0) 
		ht++;
	
	base = (**theControl).contrlRect;				// the entire control
	parts[ALL] = base;
	tWid = StringWidth((**theControl).contrlTitle);
	if(tWid)
		tWid+=5;		
	base.left+=tWid;
	parts[FONTINFO] = base;
	parts[FONTINFO].bottom = ht - f.ascent;			// baseline for DrawStr		

//----------------------------------------------------------------------------------
// part rectangles for Date portion of control; left justified	
//----------------------------------------------------------------------------------				
	
	base.top = base.bottom - ARROWHT;				// resize to arrow height	
	up = work = base;								// the up arrow rect		
	up.left += StringWidth((StringPtr)sStr) + 6;	// shift arrow to right		
	up.right = up.left + ARROWID;					// resize arrow width		
	parts[FRAMED] = down = up;						// up & down are full rects	
	up.bottom = up.top + (up.bottom - up.top)/2;	// split them in half			
	
	parts[UPDATE] = up;								// up rect is top half		
	down.top = up.bottom+1;
	parts[DOWNDATE] = down;							// down rect is bottom half	
	
	wid = StringWidth("\p88")+2;					// digit rect for 2 numbers	
	work.right = work.left + wid;
	offset = (ARROWHT - ht)/2;						// center digits in rect	
	work.bottom -= offset;
	work.top = (work.bottom-ht)/2 * 2;
	
	parts[TITLE] = work;
	parts[TITLE].left = parts[ALL].left;
	tWid = StringWidth((**theControl).contrlTitle);
	if(tWid)
		tWid+=5;
	parts[TITLE].right = parts[TITLE].left+tWid;

	parts[DT1] = parts[DATE] = work;				// the first digits rect	
	offset = CharWidth((**hDtc).dateSep)-2;			// a bit of fudge here		
	OffsetRect(&work, wid+offset,0);				// offset the other rects	
	parts[DT2] = work;
	OffsetRect(&work, wid+offset,0);
	parts[DT3] = work;
	
//----------------------------------------------------------------------------------
// part rectangles for Time portion of control; right justified		
//----------------------------------------------------------------------------------

	sStr[0] = 8;									// sample "88:88 AM"		
	sStr[3] = (**hDtc).timeSep;
	sStr[1] = sStr[2] = sStr[4] = sStr[5] = '8';
	sStr[6] = ' '; 
	sStr[7] = 'A';
	sStr[8] = 'M';
	
//----------------------------------------------------------------------------------
// build rects for up and down portion of arrow, right justified in cntrolRect	
//----------------------------------------------------------------------------------

	up = work = base;								// base rect				
	up.left = up.right - ARROWID;					// shift arrow to right		
	parts[FRAMET] = down = up;						// up & down are full rects	
	up.bottom = up.top + (up.bottom - up.top)/2;	// split them in half			
	
	parts[UPTIME] = up;								// up rect is top half		
	down.top = up.bottom+1;
	parts[DOWNTIME] = down;							// down rect is bottom half	
	
//----------------------------------------------------------------------------------
// build rects for digit portion of time string	
//----------------------------------------------------------------------------------

	offset = (ARROWHT - ht)/2;						// center digits in rect	
	work.bottom -= offset;
	work.top = (work.bottom-ht)/2 * 2;
	if(varCode & showAMPM)							// set timeRect.left		
		work.left = up.left - (StringWidth((StringPtr)sStr) + 6);
	else {
		sStr[0] = 5;
		work.left = up.left - (StringWidth((StringPtr)sStr) + 6);
	}
	work.right = work.left+wid;						// enough room for 2 digits	
	
	parts[HOURS] = work;							// hours digit rect			
	
	offset = CharWidth((**hDtc).timeSep)-2;			// a little fudge			
	OffsetRect(&work,wid+offset,0);
	parts[MINUTES] = work;							// minutes digit rect		
	
	if(varCode & showAMPM) {						// set up AM/PM rect		
		offset = CharWidth(' ')-2;					// a little fudge			
		OffsetRect(&work,wid+offset,0);
		wid = StringWidth("\pPM");
		work.right = work.left+wid+2;
	}
	else
		SetRect(&work,0,0,0,0);						// no AM/PM, so no rect		

	parts[AMPM] = work;
	
	
//----------------------------------------------------------------------------------
// shift rects around to match date format	
//----------------------------------------------------------------------------------	

	mR = parts[m+DT1];
	dR = parts[d+DT1];
	yR = parts[y+DT1];
	parts[DT1] = mR;
	parts[DT2] = dR;
	parts[DT3] = yR;
	
//----------------------------------------------------------------------------------
// shift 'em around for the time only or stacked version
//----------------------------------------------------------------------------------	
	
	if(varCode & dtStack) {					
		offset = ht + f.descent;						// shift date rects up 
		for(inx = DT1;inx<FONTINFO;inx++)
			OffsetRect(&parts[inx],0, - offset);
	}
	if(varCode & dtStack || varCode & timeOnly) {		// shift time rects left
		offset = parts[HOURS].left - parts[DATE].left;
		for(inx = FRAMET;inx<DT1;inx++)
			OffsetRect(&parts[inx], - offset, 0);
	}
	if(varCode & dtStack && varCode & showAMPM) {		// align arrows
		parts[UPTIME].left = parts[DOWNTIME].left = parts[DOWNDATE].left;
		parts[UPTIME].right = parts[DOWNTIME].right = parts[DOWNDATE].right;
		parts[FRAMET].left = parts[FRAMED].left;
		parts[FRAMET].right = parts[FRAMED].right;
	}
}

//==================================================================================
//	Convert seconds to a Pascal string containing the time
//==================================================================================
static void getTimeStr(ControlHandle theControl, short varCode, long secs, Str31 string)
{
	short		i=0;
	long		nl;
	Str31		s;
	DateTimeRec	dtr;
	dtcHandle	hDtc;
	
	hDtc = (dtcHandle)(**theControl).contrlData;
	if(!hDtc)
		return;
	
	string[i++] = 5;				// pascal string			
	
	Secs2Date(secs,&dtr);			// convert to DateTimeRec	

	nl = dtr.hour;					// convert the hours		
	if(varCode & showAMPM) {		// 12 hour display			
		if(nl > 12)
			nl-=12;
		else						// 24 hour display			
		if(nl == 0 && ((**hDtc).timeCycle == 0xFF))
			nl = 12;
	}
	NumToString(nl, s);
	if(nl > 9) {
		string[i++] = s[1];
		string[i++] = s[2];
	}
	else {
		string[i++] = '0';
		string[i++] = s[1];
	}
				
	string[i++] = (**hDtc).timeSep;
	
	nl = dtr.minute;				// convert the minutes		
	NumToString(nl, s);
	if(nl > 9) {
		string[i++] = s[1];
		string[i++] = s[2];
	}
	else {
		string[i++] = '0';
		string[i++] = s[1];
	}
	if(varCode & showAMPM) {		// fill in the AM/PM		
		string[0] = 8;				// and adjust string len	
		string[i++] = ' ';
		if(dtr.hour > 11)
			string[i++] = 'P';
		else
			string[i++] = 'A';
		string[i++] = 'M';
	}
	
}

//==================================================================================
// a "brain dead" little routine to return m, d, y order from Intl0 resource
//==================================================================================
static void getDateFmt(ControlHandle theControl, short * m, short * d, short * y)
{
	Intl0Hndl	hInt;
	dtcHandle	hDtc;
	
	hDtc = (dtcHandle)(**theControl).contrlData;
	if(!hDtc)
		return;
	
	hInt = (Intl0Hndl)IUGetIntl(0);
	
	(**hDtc).dateSep = (**hInt).dateSep;
	(**hDtc).dateOrder = (**hInt).dateOrder;
	(**hDtc).timeSep = (**hInt).timeSep;
	(**hDtc).timeCycle = (**hInt).timeCycle;
	
	switch((**hDtc).dateOrder){
		case 0:
			*m = 0;
			*d = 1;
			*y = 2;
		break;
		case 1:
			*m = 1;
			*d = 0;
			*y = 2;
		break;
		case 2:
			*m = 1;
			*d = 2;
			*y = 0;
		break;
		case 3:
			*m = 0;
			*d = 2;
			*y = 1;
		break;
		case 4:
			*m = 2;
			*d = 0;
			*y = 1;
		break;
		case 5:
			*m = 2;
			*d = 1;
			*y = 0;
		break;
	}
}
//==================================================================================
//	Convert seconds to a Pascal string containing the date, in 
//	order specified in the Date & Time Control Panel
//==================================================================================
static void getDateStr(ControlHandle theControl, long secs, Str31 string)
{
	short		i,m,d,y,dateTbl[3];
	long		nl;
	Str31		s;
	DateTimeRec	dtr;
	dtcHandle	hDtc;
	
	hDtc = (dtcHandle)(**theControl).contrlData;
	if(!hDtc)
		return;
	
	string[0] = 8;								// 8 byte pascal string		
	string[3] = string[6] = (**hDtc).dateSep;
	
	Secs2Date(secs,&dtr);						// convert to DateTimeRec	
	getDateFmt(theControl, &m, &d, &y);			// get table indices
	dateTbl[m] = dtr.month;						// and plug into local tbl	
	dateTbl[d] = dtr.day;
	dateTbl[y] = dtr.year;
	
	i = m*3+1;
	nl = dateTbl[m];							// convert the month		
	NumToString(nl, s);
	
	if(nl > 9) {
		string[i++] = s[1];
		string[i]   = s[2];
	}
	else {
		string[i++] = '0';
		string[i]   = s[1];
	}
	
	i = d*3+1;
	nl = dateTbl[d];							// convert the day			
	NumToString(nl, s);
	
	if(nl > 9) {
		string[i++] = s[1];
		string[i]   = s[2];
	}
	else {
		string[i++] = '0';
		string[i]   = s[1];
	}
	
	i = y*3+1;
	nl = dateTbl[y] - (dateTbl[y]/100 * 100);	// convert the year			
	NumToString(nl, s);
	
	if(nl > 9) {
		string[i++] = s[1];
		string[i]   = s[2];
	}
	else {
		string[i++] = '0';
		string[i]   = s[1];
	}
}

//==================================================================================
//	A simple check to see if a partCode is one of the up/down arrows
//==================================================================================
static char arrowPart (short partCode)
{
	if(partCode == UPTIME		||
		partCode == DOWNTIME	||
		partCode == UPDATE		||
		partCode == DOWNDATE)
		return true;
	return false;
}
//==================================================================================
//	create polygons for the arrow frame and both of the arrows	
//==================================================================================

void makePolys(ControlHandle theControl, short varCode)
{
	short		inx,c,v,w,b1,b2;	
	Rect		r,parts[NUMPARTS];
	dtcHandle	hDtc;
	
	hDtc = (dtcHandle)(**theControl).contrlData;
	if(!hDtc) {										// ooops!					
		return;
	}
	getRects(theControl, varCode, parts);				// rects for control parts
	OffsetRect(&parts[FRAMET], 							// adjust to 0,0			
				-parts[ALL].left, -parts[ALL].top);	
	r = parts[FRAMET];
	
	for(inx=0;inx<2;inx++) {
		(**hDtc).poly[3*inx] = OpenPoly();				// frame the arrows
		
		if((**hDtc).poly[3*inx]) {
			MoveTo(r.left+2,r.top);	
			LineTo(r.right-3, r.top);
			Line(2,3);
			LineTo(r.right-1, r.bottom-3);
			Line(-2,2);
			LineTo(r.left+2, r.bottom-1);
			Line(-2,-2);
			LineTo(r.left, r.top+3);
			Line(2,-3);
			ClosePoly();
		}
		
		c = (r.right - r.left)/2 + r.left;
		b1 = (r.right - r.left -2)/3;
		b2 = (r.right - r.left -2)/4;
		v = r.top+2;
		w = b2;
		
		(**hDtc).poly[3*inx+1] = OpenPoly();			// draw UP arrow
		
		if((**hDtc).poly[3*inx+1]) {
			MoveTo(c,v);
			Line(-b1,b1);
			Line(2,0);
			Line(0,b2);
			Line(w,0);
			Line(0,-b2);
			Line(2,0);
			Line(-b1,-b1);
			ClosePoly();
		}
			
		v = r.bottom-3;
		(**hDtc).poly[3*inx+2] = OpenPoly();			// draw DOWN arrow
		
		if((**hDtc).poly[3*inx+2]) {		
			MoveTo(c,v);
			Line(-b1,-b1);
			Line(2,0);
			Line(0,-b2);
			Line(w,0);
			Line(0,b2);
			Line(2,0);
			Line(-b1,b1);
			ClosePoly();
		}
		OffsetRect(&parts[FRAMED], 						// adjust to 0,0			
					-parts[ALL].left, -parts[ALL].top);	
		r = parts[FRAMED];
	}
}
