//-------------------------  1994 by James G. Stout -------------------------------
//	File	: cdefHSlider.c
//	Date	: April 6, 1994
//	Author	: Jim Stout
//			:
//	Purpose	: A slider CDEF modeled after the on in the Brightness Control panel
//			:
//			: see cdefHSlider.h for more detail and variation codes
//			:
//			: If you find a use for this, I'd love to know about it.  Bug reports
//			: are always interesting.
//			:
//			: Internet    : JimS@WRQ.COM(work hours, PST)
//			: AppleLink   : WRQ			(daily)
//			: CompuServe  : 73240,2052	(weekly or so)
//			: AOL         : JasG		(weekly or so)
//			: eWorld      : Jim Stout	(weekly or so)
//----------------------------------------------------------------------------------
//#define _DEBUGCDEF					// comment this to build CDEF

#include <QDOffscreen.h>
#include <GestaltEqu.h>
#include <SetUpA4.h>

#include "grayCDEF.h"
#include "colorCDEF.h"
#include "miscCDEF.h"
#include "qdCDEF.h"

#include "cdefHSlider.h"

#ifdef _DEBUGCDEF
pascal long CDmain (short, ControlHandle, short, long);

pascal long CDmain (short varCode, ControlHandle theControl, short message, long param)
#else

//==================================================================================
// CDEF entry point
//==================================================================================
pascal long main (short varCode, ControlHandle theControl, short message, long param)
#endif
{
	long	ret = 0L;
	
	RememberA0();
	SetUpA4();
	
	HLock((Handle)theControl);
	if((**theControl).contrlData)
		HLock((**theControl).contrlData);
	
	switch(message) {
		case initCntl:
			doInit(theControl);
		break;
		case dispCntl:
			doDisp(theControl);
		break;
		case drawCntl:
			if ((**theControl).contrlVis != 0 &&
				((WindowPeek)(**theControl).contrlOwner)->visible) {
				doDraw(theControl, varCode);
			}
		break;
		case testCntl:
			ret = doTest (theControl, varCode, param);
		break;
		case calcCRgns:
			RectRgn((RgnHandle)(param & 0x7fffffffL), &(**theControl).contrlRect);
		break;
		case calcCntlRgn:
		case calcThumbRgn:
			RectRgn((RgnHandle)(param), &(**theControl).contrlRect);
		break;
	}

	if((**theControl).contrlData)
		HUnlock((**theControl).contrlData);
	HUnlock((Handle)theControl);
	
	RestoreA4();
	
	return (ret);
}

//==================================================================================
//	Initialize our control data
//==================================================================================

static long doInit (ControlHandle theControl)
{
	Rect		scaleRect;
	CGrafPtr	offPort=0;
	short		pDepth;
	CDEFHandle	hCDEF;
	
//----------------------------------------------------------------------------------
// force the scale rect to the size we want. It'll be clipped to the control Rect
//----------------------------------------------------------------------------------
	
	scaleRect = (**theControl).contrlRect;
	scaleRect.right = scaleRect.left + 121;
	scaleRect.bottom = scaleRect.top + 24;
		
//----------------------------------------------------------------------------------
// force the min & max as well
//----------------------------------------------------------------------------------
	
	(**theControl).contrlMin = 0;
	(**theControl).contrlMax = 100;						// number of pixels in scale
	
	hCDEF = (CDEFHandle)NewHandle(sizeof(CDEFData));
	if(hCDEF) {
		(**hCDEF).pDepth = 1;							// filled in in doDraw()
		(**hCDEF).offPort = 0;
		(**hCDEF).scaleRect = scaleRect;
		(**hCDEF).qdVers = getQDVers();
		(**hCDEF).firstClick = 2;
		
		(**theControl).contrlData = (Handle)hCDEF;
		HLock((**theControl).contrlData);
	}
	else
		(**theControl).contrlData = 0;
}

//==================================================================================
//	Get rid of our control data
//==================================================================================

static long doDisp		(ControlHandle theControl)
{
	BitMap		*theBits;
	CDEFHandle	hCDEF;
	
	hCDEF = (CDEFHandle)(**theControl).contrlData;
	if(hCDEF) {
		if((**hCDEF).qdVers < gestalt32BitQD12) {		// offport is bitmap
			theBits = &((GrafPtr)(**hCDEF).offPort)->portBits;
			DisposePtr((*theBits).baseAddr);
			ClosePort((GrafPtr)(**hCDEF).offPort);
			DisposePtr((Ptr)(**hCDEF).offPort);
		}
		else
			DisposeGWorld((**hCDEF).offPort);			// offPort is GWorld
			
		HUnlock((Handle)hCDEF);
		DisposeHandle((Handle)hCDEF);
		(**theControl).contrlData = 0;
	}
}

//==================================================================================
//	Test to see if the mouse was pressed in the control thumb and drag accordingly
//==================================================================================

static long doTest (ControlHandle theControl, short varCode, long param)
{
	Rect	rThumb, rScale;
	Point	p;
	long	ret=0,secs;
	short	partCode;
	CDEFHandle	hCDEF;
	
	hCDEF = (CDEFHandle)(**theControl).contrlData;
	if(!hCDEF)
		return(ret);

	partCode = (**theControl).contrlHilite;
	rScale = (**hCDEF).scaleRect;
	
//----------------------------------------------------------------------------------
// get the thumb rect, calculated from control value
//----------------------------------------------------------------------------------

	getThumbRect(theControl, &rThumb, &rScale);
	InsetRect(&rScale,4,7);									// inset for active area
	
	p.h = LoWord(param);
	p.v = HiWord(param);
	
	if(PtInRect(p, &rThumb)) {								// click in thumb?
		ret = 1;
		dragThumb(theControl, varCode, param);
	}
	else
	if(PtInRect(p, &rScale)) {								// click in scale?
		if(p.h < rThumb.left)
			ret = 2;										// decrease
		else
		if(p.h > rThumb.right)
			ret = 3;										// increase
			
		if((**hCDEF).firstClick)							// seem to get called
			(**hCDEF).firstClick--;							// too often
			
		if((**hCDEF).firstClick || partCode == ret) {
			scaleClick(theControl, varCode, (short)ret);
			Delay(6L, & secs);
		}
	}
	return(ret);
}

//==================================================================================
// Respond to a mouse down in our "thumb", track the mouse to reposition the thumb,
// set a new control value and call the actionProc (if any).
//==================================================================================
static void dragThumb (ControlHandle theControl, short varCode, long param)
{
	short	newVal,maxDrag,minDrag;
	Rect	rThumb, rScale;
	Point	p,oldPt;

	rScale = (**(CDEFHandle)(**theControl).contrlData).scaleRect;
	getThumbRect(theControl, &rThumb, &rScale);
	
	p.h = oldPt.h = LoWord(param);
	p.v = HiWord(param);
	
	if(PtInRect(p, &rThumb)) {
		maxDrag = rScale.right - 4 - (rThumb.right - p.h);
		minDrag = rScale.left + 4 + (p.h - rThumb.left);
		while (StillDown()) {
			if(oldPt.h != p.h) {							// mouse moved
				OffsetRect(&rThumb, (p.h-oldPt.h), 0);		// move the thumb
				if(rThumb.left < rScale.left+5) {			// pin to left of scale
					OffsetRect(&rThumb, rScale.left - rThumb.left+5, 0); 
				}
				else
				if(rThumb.right > rScale.right-5) {			// pin to right of scale
					OffsetRect(&rThumb, rScale.right - rThumb.right-5, 0); 
				}
				newVal =  -(rScale.left - rThumb.left + 5);
				if(newVal != (**theControl).contrlValue) {
					(**theControl).contrlValue = newVal;
					doDraw(theControl, varCode);
					if((**theControl).contrlAction != nil)
						CallPascalL(theControl, 1,(**theControl).contrlAction);
				}
				if(p.h > minDrag && p.h < maxDrag)			// track move
					oldPt.h = p.h;	
			}
			GetMouse(&p);
		}
	}
}

//==================================================================================
// Adjust the control after detecting a click in the scale instead of the thumb -
// set a new control value and call the actionProc (if any).
//==================================================================================
static void scaleClick (ControlHandle theControl, short varCode, short partCode)
{
	Rect	rThumb, rScale;
	short	newVal;

	rScale = (**(CDEFHandle)(**theControl).contrlData).scaleRect;
	getThumbRect(theControl, &rThumb, &rScale);
	
	if(partCode == 2) {
		OffsetRect(&rThumb, -1, 0);
	}
	else 
	if(partCode == 3) {
 		OffsetRect(&rThumb, 1, 0);
	}	
	if(rThumb.left < rScale.left+5)							// pin to left of scale
		OffsetRect(&rThumb, rScale.left - rThumb.left+5, 0); 
	else
	if(rThumb.right > rScale.right-5)						// pin to right of scale
		OffsetRect(&rThumb, rScale.right - rThumb.right-5, 0);
		
	newVal =  -(rScale.left - rThumb.left + 5);
	
	if(newVal != (**theControl).contrlValue) {
		(**theControl).contrlValue = newVal;
		doDraw(theControl, varCode);
		if((**theControl).contrlAction != nil)
			CallPascalL(theControl, partCode,(**theControl).contrlAction);
	}
}

//==================================================================================
//	Calculate the thumb location from the control value
//==================================================================================

static void getThumbRect(ControlHandle theControl, Rect *rThumb, Rect *rScale)
{
	short	h;
	
	SetRect(rThumb, 0, 0, 11, 24);
	h = (**theControl).contrlValue + (rScale)->left + 5;
	OffsetRect(rThumb, h, (rScale)->top);
}

//==================================================================================
//	Draw both the scale and thumb into one bitmap.  We've got the scale & thumb
//  stored as bitmap data in a char array.
//==================================================================================

Boolean drawParts (ControlHandle theControl, short varCode, Rect *rThumb)
{
#include "cdefHSliderBits.c"						// bitmap data

	Rect		partRect;
	BitMap		bits;
	Boolean		inColor = false, haveGray = false;
	GrafPtr		offPort;
	RGBColor	saveFore,saveBack,grayColor,tingeColor;
	RGBColor	scaleColor = {56797, 56797, 56797};
	Pattern		pat = { 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA};
	CDEFHandle	hCDEF;
	
	hCDEF = (CDEFHandle)(**theControl).contrlData;
	if(hCDEF) {
		offPort = (GrafPtr)(**hCDEF).offPort;
	  	if((**hCDEF).pDepth > 2) {
	  		inColor = true;
	  		saveColors(&saveFore, &saveBack);	
	  		setPartColor(theControl, cFrameColor, true);
	  	}
		if((**theControl).contrlHilite == 0xFF) {	// inactive control
			if(inColor) {							// try to draw it in
				haveGray = getGray(&grayColor);		// gray
				if(haveGray)
					RGBForeColor(&grayColor);
			}
		}
		
//----------------------------------------------------------------------------------
// draw the scale
//----------------------------------------------------------------------------------

		SetRect(&partRect, 0, 0, 121, 24);			// construct a bitmap	
		bits.bounds = partRect;						// on the fly
		bits.rowBytes = 16;							// for the scale
		bits.baseAddr = hScale;						// in data array
		
		CopyBits(&bits,								// from bitmap
				&(offPort)->portBits,				// to offscreen port
				&bits.bounds,						// from rect
				&bits.bounds,					 	// to rect
				srcCopy, nil);
		
		if(varCode & ctl3D && inColor) {
	  		InsetRect(&partRect, 3, 8);
	  		GetForeColor(&saveFore);
			RGBForeColor(&scaleColor);
			if(haveGray) {							// inactive
				if(getGray(&grayColor))
					RGBForeColor(&grayColor);
			}
			PaintRoundRect(&partRect,4,4);

//----------------------------------------------------------------------------------
// draw the 3D shadow in a grayed cTingeLight
//----------------------------------------------------------------------------------

	  		setPartColor(theControl, cTingeLight, true);
	  		GetForeColor(&tingeColor);
	  		tingeColor.red-=13107;
	  		tingeColor.green-=13107;
	  		tingeColor.blue-=13107;
	  		RGBForeColor(&tingeColor);
			if(haveGray) {							// inactive
				if(getGray(&grayColor))
					RGBForeColor(&grayColor);
			}
	  		MoveTo(partRect.left+1, partRect.bottom-1);
	  		LineTo(partRect.right-2, partRect.bottom-1);
	  		MoveTo(partRect.right-1, partRect.bottom-2);
	  		LineTo(partRect.right-1, partRect.top+1);
	  		MoveTo(partRect.right-2, partRect.bottom-2);
	  		LineTo(partRect.right-2, partRect.bottom-2);

	  		RGBForeColor(&saveFore);
		}
		else
	  	if(varCode & scaleNoFill) {
	  		InsetRect(&partRect, 3, 8);
	  		EraseRoundRect(&partRect, 4,4);
	  	}
	  	else
	  	if(varCode & scaleGray) {
	  		InsetRect(&partRect, 3, 8);
#ifdef dangerousPattern
	  		FillRoundRect(&partRect, 4, 4, pat);
#else
	  		FillRoundRect(&partRect, 4, 4, &pat);
#endif
	  	}

//----------------------------------------------------------------------------------
// draw the thumb
//----------------------------------------------------------------------------------
	
		SetRect(&partRect, 0, 0, 11, 24);			// construct a bitmap
		bits.bounds = partRect;						// on the fly
		bits.rowBytes = 2;							// for the thumb
		bits.baseAddr = hThumb;						// in data array
		
		CopyBits(&bits,								// from bitmap
				&(offPort)->portBits,				// to onscreen port
				&partRect,							// from rect
				rThumb,			 					// to rect
				srcCopy, nil);
				
	  	if(varCode & ctl3D && inColor) {	
	  		RGBForeColor(&scaleColor);
			if(haveGray) {							// inactive
				if(getGray(&grayColor))
					RGBForeColor(&grayColor);
			}
	  		InsetRect(rThumb, 3, 1);
	  		PaintRect(rThumb);

	  		MoveTo((*rThumb).left-2, (*rThumb).bottom-2);
	  		LineTo((*rThumb).left-2, (*rThumb).top+1);
	  		MoveTo((*rThumb).right+1, (*rThumb).bottom-2);
	  		LineTo((*rThumb).right+1, (*rThumb).top+1);

//----------------------------------------------------------------------------------
// draw the 3D shadow in a grayed cTingeColor
//----------------------------------------------------------------------------------

	  		RGBForeColor(&tingeColor);
			if(haveGray) {							// inactive
				if(getGray(&grayColor))
					RGBForeColor(&grayColor);
			}
	  		MoveTo((*rThumb).left, (*rThumb).bottom-1);
	  		LineTo((*rThumb).right-1, (*rThumb).bottom-1);
	  		LineTo((*rThumb).right-1, (*rThumb).top);
	  		MoveTo((*rThumb).right-2, (*rThumb).bottom-2);
	  		LineTo((*rThumb).right-2, (*rThumb).bottom-2);
	  	}
	  	if(inColor) {
	  		restoreColors(&saveFore, &saveBack);
	  	}
  	}
  	return(haveGray);
}

//==================================================================================
//	Draw the control, it is made up from the scale bitmap and the thumb bitmap
//	which are assembled into an offscreen bitmap in "drawParts".
//==================================================================================

static void doDraw (ControlHandle theControl, short varCode)
{
	short			partCode;
	Rect			offRect,rThumb,rScale;
	RgnHandle		clipRgn;
	BitMap			*offBits;
	PixMapHandle	pmHdl;
	CGrafPtr		savePort;
	GDHandle		saveGDev;
	PenState		savePen;
	Boolean			inactive = false, haveGray = false, haveGW = false, inColor = false;
	RGBColor		saveFore,saveBack;
	RGBColor		rgbBlack = {0,0,0};
	RGBColor		rgbWhite = {-1,-1,-1};
	CDEFHandle		hCDEF;
	
	hCDEF = (CDEFHandle)(**theControl).contrlData;
	if(hCDEF) {
		
//----------------------------------------------------------------------------------
// first, create or update our GWorld	
//----------------------------------------------------------------------------------
	
		(**hCDEF).pDepth = getOff(&(**hCDEF).offPort, &(**theControl).contrlRect);
		
		if((**hCDEF).pDepth == 0) {								// oh oh				
			return;										
		}
		
		if((**hCDEF).qdVers >= gestalt32BitQD12) {				// have a GWorld
			haveGW = true;
			pmHdl = getLockedPixels(&(**hCDEF).offPort, (**hCDEF).qdVers);
			if(!pmHdl)
				return;											// nuts
		}
	
		clipRgn = NewRgn();
		GetClip(clipRgn);
		ClipRect(&(**theControl).contrlRect);
		
//----------------------------------------------------------------------------------
// Position the thumb rect
//----------------------------------------------------------------------------------
		
		SetRect(&rThumb, 0, 0, 11, 24);
		OffsetRect(&rThumb, (**theControl).contrlValue+5,0);
		
		SetRect(&offRect, 0, 0, 121, 24);
		rScale = (**hCDEF).scaleRect;
		
//----------------------------------------------------------------------------------
// Initialize and set for drawing into offScreen port
//----------------------------------------------------------------------------------

		if((**theControl).contrlHilite == 0xFF)				// inactive control
			inactive = true;
			
		if((**hCDEF).pDepth > 2) {							// save onscreen	
			inColor = true;									// colors
			saveColors(&saveFore, &saveBack);
		}
		
		if(haveGW) {										// we have a pixMap		
			GetGWorld(&savePort, &saveGDev);
			SetGWorld((**hCDEF).offPort, nil);		
			offBits = (BitMap *)*pmHdl;
		}
		else {												// we have a bitMap	
			GetPort((GrafPtr*)&savePort);
			SetPort((GrafPtr)(**hCDEF).offPort);		
			offBits = (BitMap *)&((GrafPtr)(**hCDEF).offPort)->portBits;
		}
		
		if(inColor) {										// need same colors
			RGBForeColor(&saveFore);						// offscreen
			RGBBackColor(&saveBack);
		}
		
//----------------------------------------------------------------------------------
// draw the control parts into the offscreen port
//----------------------------------------------------------------------------------
	
		haveGray = drawParts (theControl, varCode, &rThumb);
		
//----------------------------------------------------------------------------------
// restore onscreen port & set colors to b&w 
//----------------------------------------------------------------------------------

		if(haveGW) {
			SetGWorld(savePort, saveGDev);
		} else {
			SetPort((GrafPtr)savePort);
		}
		if(inColor) {
			RGBForeColor(&rgbBlack);
			RGBBackColor(&rgbWhite);
		}

//----------------------------------------------------------------------------------
// now copy it to the onscreen port and restore our colors
//----------------------------------------------------------------------------------
	
		CopyBits(offBits,
				&((GrafPtr)savePort)->portBits,
				&offRect,
				&rScale,
				srcCopy, nil);
		
		if((**hCDEF).qdVers >= gestalt32BitQD13)
			UnlockPixels(pmHdl);
		
		if(inColor) {
			restoreColors(&saveFore, &saveBack);
		}
		
//----------------------------------------------------------------------------------
// if we failed to get an inactive gray color, do it the old way
//----------------------------------------------------------------------------------

		if(inactive && !haveGray) {							// use the old way of drawing
			GetPenState(&savePen);							// an inactive control
			PenMode(patBic);
			PenPat( (ConstPatternParam) "\xAA\x55\xAA\x55\xAA\x55\xAA\x55");
			PaintRect(&(**theControl).contrlRect);
			SetPenState(&savePen);
		}
		SetClip(clipRgn);
		DisposeRgn(clipRgn);
	}
}