//-------------------------  1992-1994 by James G. Stout --------------------------
// File		: cdefSpinner.c
// Date		: 24 October 1992
// Author	: Jim Stout
//			:
// Purpose	: a "Spinner Control" CDEF that allows adjustment of a
//			: number via an up/down spinner - a "little arrow" control.
//			:
//			: see cdefSpinner.h for more detail and variation codes
//			:
//			: If you find a use for this, I'd love to know about it.  Bug reports
//			: are always interesting.
//			:
//			: Internet    : JimS@WRQ.COM(work hours, PST)
//			: AppleLink   : WRQ			(daily)
//			: CompuServe  : 73240,2052	(weekly or so)
//			: AOL         : JasG		(weekly or so)
//			: eWorld      : Jim Stout	(weekly or so)
//----------------------------------------------------------------------------------
//#define _DEBUGCDEF 1

#include <QDOffscreen.h>
#include <GestaltEqu.h>
#include <SetUpA4.h>

#include "cdefSpinner.h"

#include "grayCDEF.h"
#include "colorCDEF.h"
#include "miscCDEF.h"
#include "qdCDEF.h"


#ifdef _DEBUGCDEF
pascal long CDmain (short, ControlHandle, short, long);

pascal long CDmain (short varCode, ControlHandle theControl, short message, long param)
#else

//==================================================================================
// CDEF entry point
//==================================================================================

pascal long main (short varCode, ControlHandle theControl, short message, long param)
#endif
{
	long		ret = 0L;
	short		txF,txS;
	GrafPtr		thisPort;
	hSpinData	hSpin;
		
	RememberA0();
	SetUpA4();
	
	HLock((Handle)theControl);
	
//----------------------------------------------------------------------------------
// make sure we've got the correct font	
//----------------------------------------------------------------------------------

	if(!(varCode & useWindFont)) {			// don't use the window font	
		GetPort(&thisPort);
		txF = thisPort->txFont;
		txS = thisPort->txSize;
		thisPort->txFont = SysFontFam;		// use the System font & size	
		thisPort->txSize = SysFontSize;
	}

//----------------------------------------------------------------------------------
// get our private data	
//----------------------------------------------------------------------------------

	if((**theControl).contrlData) {
		HLock((**theControl).contrlData);
		hSpin = (hSpinData)(**theControl).contrlData;
	}
	
//----------------------------------------------------------------------------------
// handle the standard control messages	
//----------------------------------------------------------------------------------

	switch(message) {
		case initCntl:
			doInit(theControl, varCode, param);
		break;
		case dispCntl:
			doDisp(theControl, varCode, param);
		break;
		case testCntl:		
			if((**theControl).contrlHilite != 0xFF)			// only if active
				ret = doTest (theControl, varCode, param);
		break;
		case drawCntl:
			if ((**theControl).contrlVis != 0 &&
				((WindowPeek)(**theControl).contrlOwner)->visible) {
				doDraw(theControl, varCode);
			}
		break;
		case calcCRgns:
			RectRgn((RgnHandle)(param & 0x7fffffffL), &(**theControl).contrlRect);
		break;
		case calcCntlRgn:
		case calcThumbRgn:
			RectRgn((RgnHandle)(param), &(**theControl).contrlRect);
		break;
	}
		
//----------------------------------------------------------------------------------
// restore window font & size info	
//----------------------------------------------------------------------------------
	
	if(!(varCode & useWindFont)) {
		thisPort->txFont = txF;
		thisPort->txSize = txS;
	}

	if((**theControl).contrlData)
		HUnlock((**theControl).contrlData);
	HUnlock((Handle)theControl);
	
	RestoreA4();
	return (ret);
}
 
//==================================================================================
//	initialize our private data and save in contrlData field.
//==================================================================================

static long doInit (ControlHandle theControl, short varCode, long param)
{
	hSpinData	hSpin;
	
	hSpin = (hSpinData) NewHandle(sizeof(spinData));
	if(hSpin) {

//----------------------------------------------------------------------------------	
//  If the control refCon is non-zero, the LoWord is the increment
//	to use and the HiWord is a DITL item number for an editText item
//	that we will set with the contrlValue in updateValue.
//----------------------------------------------------------------------------------
	
		(**hSpin).currIncr	= 1;
		(**hSpin).increment	= 1;
		(**hSpin).itemNum 	= 0;
		
		if((**theControl).contrlRfCon) {
			(**hSpin).increment = LoWord((**theControl).contrlRfCon);
			if((**hSpin).increment < 1 ||
				(**hSpin).increment > 1000) {						
				(**hSpin).increment = 1;
			}
			(**hSpin).currIncr = (**hSpin).increment;
			if((*(WindowPeek)(**theControl).contrlOwner).windowKind == dialogKind)
				(**hSpin).itemNum  = HiWord((**theControl).contrlRfCon);
			if((**hSpin).itemNum < 0 || (**hSpin).itemNum > 100)					
				(**hSpin).itemNum = 0;
		}
		(**hSpin).bumpit	= -3;
				
		(**hSpin).offPort 	= 0;					// offPort & 
		(**hSpin).pDepth 	= 0;					// pDepth filled in in doDraw()
		(**hSpin).firstClick = 2;
		(**hSpin).qdVers 	= getQDVers();
		
		(**hSpin).poly[0] = 0;						// filled in in makePolys()
		(**hSpin).poly[1] = 0;
		(**hSpin).poly[2] = 0;
		(**theControl).contrlData = (Handle)hSpin;
		HLock((Handle)hSpin);
		
		makePolys(theControl, varCode);				// draw arrows & frame
	}
	else
		(**theControl).contrlData = 0;
}
		
//==================================================================================
//	dispose of our private data (offscreen bitmap)
//==================================================================================

static long doDisp (ControlHandle theControl, short varCode, long param)
{
	hSpinData	hSpin;
	BitMap		*theBits;
	short		inx;
	
	hSpin = (hSpinData)(**theControl).contrlData;
	if(hSpin) {
		if((**hSpin).qdVers < gestalt32BitQD12) {	// offport is bitmap	
			theBits = &((GrafPtr)(**hSpin).offPort)->portBits;
			DisposePtr((*theBits).baseAddr);
			ClosePort((GrafPtr)(**hSpin).offPort);
			DisposePtr((Ptr)(**hSpin).offPort);
		}
		else
			DisposeGWorld((**hSpin).offPort);		// offPort is GWorld	
		for(inx=0;inx<3;inx++) {
			if((**hSpin).poly[inx])
				KillPoly((**hSpin).poly[inx]);
		}
		HUnlock((Handle)hSpin);
		DisposeHandle((Handle)hSpin);
		(**theControl).contrlData = 0;
	}
}

//==================================================================================
//	create polygons for the VERTICAL arrow frame and the up and down arrows	
//==================================================================================

void makePolys(ControlHandle theControl, short varCode)
{
	short		c,v,w,b1,b2;	
	Rect		r,parts[NUMPARTS];
	hSpinData	hSpin;

	hSpin = (hSpinData)(**theControl).contrlData;
	if(!hSpin) {										// ooops!					
		return;
	}
	getRects(theControl, varCode, parts);				// rects for control parts
	OffsetRect(&parts[FRAME], 							// adjust to 0,0			
				-parts[ALL].left, -parts[ALL].top);	
	r = parts[FRAME];
	
	(**hSpin).poly[0] = OpenPoly();						// draw frame for arrows
	
	if((**hSpin).poly[0]) {
		MoveTo(r.left+2,r.top);	
		LineTo(r.right-3,r.top);
		Line(2,3);
		LineTo(r.right-1,r.bottom-3);
		Line(-2,2);
		LineTo(r.left+2,r.bottom-1);
		Line(-2,-2);
		LineTo(r.left,r.top+3);
		Line(2,-3);
		ClosePoly(); // poly[0]
	}
	
	if(varCode & horizArrows) {
		v = (r.bottom - r.top)/2 + r.top;
		b1 = (r.bottom - r.top -2)/3;
		b2 = (r.bottom - r.top -2)/4;
		c = r.right -3;
		w = b2;
		if(varCode & bigArrows) {
			c--;
			w++;
		}
	}
	else {
		c = (r.right - r.left)/2 + r.left;
		b1 = (r.right - r.left -2)/3;
		b2 = (r.right - r.left -2)/4;
		v = r.top+2;
		w = b2;
		if(varCode & bigArrows) {
			v++;
			w++;
		}
	}
	(**hSpin).poly[1] = OpenPoly();						// draw UP arrow
	
	if((**hSpin).poly[1]) {
		if(varCode & horizArrows) {
			MoveTo(c,v);								// apex of arrow
			Line(-b1,-b1);								// upper slope of arrow head
			Line(0,2);									// down to shaft
			Line(-b2,0);								// left to end
			Line(0,w);									// down for shaft end
			Line(b2,0);									// botttom edge of shaft
			Line(0,2);									// bottom of arrow
			Line(b1,-b1);								// lower slope of arrow head
		}
		else {
			MoveTo(c,v);								// see above
			Line(-b1,b1);
			Line(2,0);
			Line(0,b2);
			Line(w,0);
			Line(0,-b2);
			Line(2,0);
			Line(-b1,-b1);
		}
		ClosePoly(); // poly[1]
	}
	
	(**hSpin).poly[2] = OpenPoly();						// draw LEFT arrow
	
	if((**hSpin).poly[2]) {		
		if(varCode & horizArrows) {	
			c = r.left+2;
			if(varCode & bigArrows)
				c++;
			
			MoveTo(c,v);								// see above
			Line(b1,-b1);
			Line(0,2);
			Line(b2,0);
			Line(0,w);
			Line(-b2,0);
			Line(0,2);
			Line(-b1,-b1);
		}
		else {											// draw DOWN arrow
			v = r.bottom-3;
			if(varCode & bigArrows)
				v--;
				
			MoveTo(c,v);								// see above
			Line(-b1,-b1);
			Line(2,0);
			Line(0,-b2);
			Line(w,0);
			Line(0,b2);
			Line(2,0);
			Line(-b1,b1);
		}
		ClosePoly(); // poly[2]
	}
}

//==================================================================================
//	check for a mouseDown in our control. Actually, we do all of our 
//	tracking, drawing and updating of the control here.	
//==================================================================================

static long doTest (ControlHandle theControl, short varCode, long param)
{
	Point		p;
	short		inx,partCode;
	long		ret = 0;
	Rect		parts[NUMPARTS];
	hSpinData	hSpin;
	
	hSpin = (hSpinData)(**theControl).contrlData;
	if(!hSpin)
		return(ret);
	
	p.h = LoWord(param);							// get the mouse hit point		
	p.v = HiWord(param);		
	
	getRects(theControl, varCode, parts);
	
	partCode = (**theControl).contrlHilite;
	
	if((**hSpin).firstClick)						// Track Control will call us
		(**hSpin).firstClick--;						// again
	
	for(inx = UPARROW;inx <= DOWNARROW;inx++) {		
		if(PtInRect(p,&parts[inx])) {
			if(inx == partCode || (**hSpin).firstClick) {
				updateValue(theControl, inx);		// change value
				doDraw(theControl, varCode);		// draw it
				accelerate(theControl);				// speed up if needed
			}
			else {									// reset increment
				(**hSpin).currIncr = (**hSpin).increment;	
				(**hSpin).bumpit = -3;
			}
			ret = inx;
		}
	}
	return(ret);	
}

//==================================================================================
//	update the controlValue and pause a bit
//==================================================================================

static void updateValue (ControlHandle theControl, short partCode)
{
	short		t,incr1,incr2,incr3;
	Handle		hEdit;
	Rect		r;
	long		secs;
	Str31		s;
	hSpinData	hSpin;
	
	hSpin = (hSpinData)(**theControl).contrlData;
	if(!hSpin)
		return;
			
//----------------------------------------------------------------------------------
// Update the controValue
//----------------------------------------------------------------------------------
	if(partCode == UPARROW) {
		if((**theControl).contrlValue <= (**theControl).contrlMax - (**hSpin).currIncr) 
			(**theControl).contrlValue += (**hSpin).currIncr;
		else
			(**theControl).contrlValue = (**theControl).contrlMax;
	}
	else {
		if((**theControl).contrlValue >= (**theControl).contrlMin + (**hSpin).currIncr)
			(**theControl).contrlValue -= (**hSpin).currIncr;
		else
			(**theControl).contrlValue = (**theControl).contrlMin;
	}
	
//----------------------------------------------------------------------------------
// if we got an Edit Text item number in the RefCon, put the text
//	into the edit item		
//----------------------------------------------------------------------------------							
	
	if((**hSpin).itemNum) {
		GetDItem((**theControl).contrlOwner, 		// get handle to text item	
					(**hSpin).itemNum, &t, &hEdit, &r);
		if(hEdit && (t & editText))	{				// is it valid ?
			NumToString((long)(**theControl).contrlValue, s);
			SetIText(hEdit,s);
			SelIText((**theControl).contrlOwner, (**hSpin).itemNum, 32767, 32767);
		}
		else
			(**hSpin).itemNum = 0;
	}
	
//----------------------------------------------------------------------------------
// now, pause a bit to let the user catch up
//----------------------------------------------------------------------------------	

	incr1 = (**hSpin).increment;
	if(incr1 == 1)
		incr2 = 10;
	else
		incr2 = incr1*incr1;
	incr3 = incr2*incr2;
	
	if((**hSpin).currIncr == incr1)
		Delay(10L, &secs);
	else
	if((**hSpin).currIncr == incr2)
		Delay(12L, &secs);
	else
	if((**hSpin).currIncr == incr3)
		Delay(16L, &secs);
}

//==================================================================================
//	process the drawCntl message	
//==================================================================================
 
void doDraw (ControlHandle theControl, short varCode)
{
	RgnHandle		c;
	Boolean			inactive=false,inColor=false,bgInColor=false;
	Boolean			haveGW=false,haveGray=false,drawText=true;
	PenState		penSt;
	short			partCode=0,val,wid,h,v;
	Str31			s;
	Rect			r,parts[NUMPARTS];
	BitMap			*offBits;
	PixMapHandle	pmHdl;
	CGrafPtr		savePort;
	GDHandle		saveGDev;
	RGBColor		saveFore,saveBack;
	RGBColor		rgbBlack = {0,0,0};
	RGBColor		rgbWhite = {-1,-1,-1};
	hSpinData		hSpin;

	hSpin = (hSpinData)(**theControl).contrlData;
	if(!hSpin) {											// ooops!					
		return;
	}

//----------------------------------------------------------------------------------		
// Get drawing rects, offPort and lock down the pixels
//----------------------------------------------------------------------------------	

	getRects(theControl, varCode, parts);					// rects for control parts
	
	(**hSpin).pDepth = getOff(&(**hSpin).offPort, &parts[ALL]);
		
	if((**hSpin).pDepth == 0) {								// oh oh					
		return;										
	}

	if((**hSpin).qdVers >= gestalt32BitQD12) {				// have a GWorld
		haveGW = true;
			pmHdl = getLockedPixels(&(**hSpin).offPort, (**hSpin).qdVers);
			if(!pmHdl)
				return;										// nuts
	}
		
//----------------------------------------------------------------------------------	
// initialize our drawing stuff	
//----------------------------------------------------------------------------------	
	
	if((**theControl).contrlHilite == 0xFF)
		inactive = true;
	else
		partCode = (**theControl).contrlHilite;
		
	GetPenState(&penSt);	
	c = NewRgn();										// set new clipping region	
	GetClip(c);
	ClipRect(&parts[ALL]);								// clip to our rect			
	
	if ((**hSpin).pDepth > 2) {
		inColor = true;
		saveColors(&saveFore, &saveBack);
		bgInColor = true;
		if(saveBack.red == 65535 &&						// is bg white?
			saveBack.green == 65535 &&
			saveBack.blue == 65535)
			bgInColor = false;
	}
#ifdef NOEMBOSS
	bgInColor = false;
#endif	

//----------------------------------------------------------------------------------	
// set our offScreen port so we can draw to it	
//----------------------------------------------------------------------------------
	if(haveGW) {										// we have a pixMap
		GetGWorld(&savePort, &saveGDev);
		SetGWorld((**hSpin).offPort, nil);
		offBits = (BitMap *)*pmHdl;
	}
	else {												// we have a bitMap	
		GetPort((GrafPtr*)&savePort);
		SetPort((GrafPtr)(**hSpin).offPort);		
		offBits = (BitMap *)&((GrafPtr)(**hSpin).offPort)->portBits;
	}

	if(inColor) {										// set colors in offPort	
		RGBForeColor(&saveFore);
		RGBBackColor(&saveBack);
	}
	else {
		ForeColor(blackColor);
		BackColor(whiteColor);
	}
	
	if(varCode & useWindFont) {							// set font in offPort	
		TextFont(savePort->txFont);
		TextSize(savePort->txSize);
	}
		
//----------------------------------------------------------------------------------
// Finally, we are ready to start drawing...
//
// draw control title into offPort
//----------------------------------------------------------------------------------

	EraseRect(&offBits->bounds);
	
	wid = parts[ALL].right - parts[ALL].left;
	if(varCode & bigArrows && wid == ARROWID1)
		drawText = false;
	else
	if(wid == ARROWID)
		drawText = false;
			
	if(drawText) {
		OffsetRect(&parts[TITLE], 							// adjust to 0,0			
					-parts[ALL].left, -parts[ALL].top);
		h = parts[TITLE].left +1;
		v = parts[TITLE].bottom -parts[BASELINE].bottom;
		
		if(bgInColor && varCode & ctl3D) {					// emboss the title
			RGBForeColor(&rgbWhite);
			MoveTo(h+1, v+1);
			DrawString((**theControl).contrlTitle);
		}
		if(inColor) {
			setPartColor(theControl, cTextColor, true);
			if(inactive) {									// draw in gray	
				if(haveGrayText()) {
					TextMode(grayishTextOr);
					haveGray = true;
				}
			}
		}
		MoveTo(h, v);
		DrawString((**theControl).contrlTitle);
		TextMode(srcOr);
	
//----------------------------------------------------------------------------------
// draw control value into offPort after converting to a string 
//----------------------------------------------------------------------------------

		val = (**theControl).contrlValue;
		NumToString((long)val, s);
		wid = StringWidth(s) + 6;
		h = parts[TITLE].right - wid;
		
		if(bgInColor && varCode & ctl3D) {					// emboss the value
			RGBForeColor(&rgbWhite);
			MoveTo(h+1, v+1);
			DrawString(s);
		}
		if(inColor) {
			setPartColor(theControl, cTextColor, true);
			if(haveGray)									// draw in gray
				TextMode(grayishTextOr);
		}
		MoveTo(h, v);
		DrawString(s);
		TextMode(srcOr);
	}
	else
	if(inactive && inColor)
		haveGray = haveGrayText();
	
//----------------------------------------------------------------------------------
// draw the arrows into the offPort	
//----------------------------------------------------------------------------------
			
	drawArrows(theControl, varCode, partCode, inactive, inColor);
	
//----------------------------------------------------------------------------------
// now blit the image from offscreen image to onscreen 
//----------------------------------------------------------------------------------

	if(haveGW) {										// back to onscreen port	
		SetGWorld(savePort, saveGDev);
	} else {
		SetPort((GrafPtr)savePort);
	}
	if(inColor) {
		RGBForeColor(&rgbBlack);
		RGBBackColor(&rgbWhite);
	}
	
	CopyBits(offBits,									// from bitmap				
			&((GrafPtr)savePort)->portBits,		 		// to onscreen port			
			&(**hSpin).offPort->portRect,				// from rect				
			&parts[ALL], 								// to rect					
			srcCopy, nil);
	
	if((**hSpin).qdVers >= gestalt32BitQD13)
		UnlockPixels(pmHdl);
			
	if(inColor) {
		restoreColors(&saveFore, &saveBack);
	}
	
//----------------------------------------------------------------------------------
// if inactive & System 6.0.x, gray out the control the old way	
//----------------------------------------------------------------------------------

	if(inactive && !haveGray) {	
		PenPat( (ConstPatternParam) "\xAA\x55\xAA\x55\xAA\x55\xAA\x55");
		PenMode(patBic);
		PaintRect(&parts[ALL]);		
	}
	SetClip(c);	
	DisposeRgn(c);
	SetPenState(&penSt);
}

//==================================================================================
//	Draw the actual arrow control image.
//==================================================================================

static void drawArrows(ControlHandle theControl, short varCode, short partCode, 
						Boolean inactive, Boolean inColor)
{
	short		inx;
	Rect		arrowRect,parts[NUMPARTS];
	RGBColor	rgbBlack = {0,0,0};
	RGBColor	rgbWhite = {-1,-1,-1};
	RGBColor	inGray = {61166, 61166, 61166};
	RGBColor	bgGray = {56797, 56797, 56797};
	RGBColor	fgGray = {30583, 30583, 30583};
	RGBColor	grayColor;
	hSpinData	hSpin;
	
	hSpin = (hSpinData)(**theControl).contrlData;
	
	getRects(theControl, varCode, parts);		// rects for control parts
	
//----------------------------------------------------------------------------------
// Frame the arrow box then fill & shadow if 3D
//----------------------------------------------------------------------------------

	if((**hSpin).poly[0]) {						// this poly is the frame
		if(inColor) {
			if(varCode & ctl3D) {				// 3D filled in gray
				if(inactive)
					RGBForeColor(&inGray);
				else
					RGBForeColor(&bgGray);
					
				PaintPoly((**hSpin).poly[0]);	// fill the box
				
				if(inactive)					// framed in gray or black
					RGBForeColor(&fgGray);
				else
					RGBForeColor(&rgbBlack);
			}
			else {								// use cctb frame color
				setPartColor(theControl, cFrameColor, true);
				if(inactive && getGray(&grayColor))
					RGBForeColor(&grayColor);
			}
		}
		FramePoly((**hSpin).poly[0]);
		
//----------------------------------------------------------------------------------
// draw shadow if 3D arrow
//----------------------------------------------------------------------------------

		if(inColor && varCode & ctl3D && !inactive) {
			arrowRect = parts[FRAME];
			OffsetRect(&arrowRect, -parts[ALL].left, -parts[ALL].top);
			RGBForeColor(&fgGray);
			MoveTo(arrowRect.left+2, arrowRect.bottom-2);
			LineTo(arrowRect.right-3, arrowRect.bottom-2);
			LineTo(arrowRect.right-3, arrowRect.bottom-3);
			LineTo(arrowRect.right-2, arrowRect.bottom-3);
			LineTo(arrowRect.right-2, arrowRect.top+2);
			setPartColor(theControl, cFrameColor, true);
		}
	}
	
//----------------------------------------------------------------------------------
// The small, non-3D arrows are hilited by filling the top/bottom half
// of the frame with the bgColor.  Then we fill/paint the arrow poly in white.
//----------------------------------------------------------------------------------

	if(!(varCode & ctl3D) || !inColor) {
		if(partCode && !(varCode & bigArrows)) {
			arrowRect = parts[partCode];
			OffsetRect(&arrowRect, -parts[ALL].left, -parts[ALL].top);
			if(varCode & horizArrows) {
				if(partCode == UPARROW)
					MoveTo(arrowRect.left, arrowRect.top+1);
				else
					MoveTo(arrowRect.left+1,arrowRect.top+1);
				for(inx=0;inx<8;inx++) {
					Line(0,8);
					Move(1,-8);
				}
			}
			else {
				if(partCode == UPARROW)
					MoveTo(arrowRect.left+1, arrowRect.top+1);
				else
					MoveTo(arrowRect.left+1,arrowRect.top-1);
				for(inx=0;inx<8;inx++) {
					Line(8,0);
					Move(-8,1);
				}
			}
		}
	}

//----------------------------------------------------------------------------------
// Draw both the top & bottom arrows (poly[1] & poly[2])
//----------------------------------------------------------------------------------

	for(inx=1;inx<3;inx++) {
		if((**hSpin).poly[inx]) {						// Need the poly!
//----------------------------------------------------------------------------------
// First check to see if we are drawing a hilited arrow
//----------------------------------------------------------------------------------
			if(partCode == inx+1) {						// arrow is hilited
				if(varCode & bigArrows) {
					if(inColor)
						RGBForeColor(&rgbBlack);		// always black
				}
				else {									// small arrow
					if(inColor) {
						if(varCode & ctl3D)				// drawn in black
							RGBForeColor(&rgbBlack);	// for 3D
						else
							RGBForeColor(&rgbWhite);	// white for non-3D	
					}
					else								// b&w hilited arrow
						ForeColor(whiteColor);			// must draw white
				}
				PaintPoly((**hSpin).poly[inx]);			// draw the arrow
			}
//----------------------------------------------------------------------------------
// Nope, it is not hilited
//----------------------------------------------------------------------------------
			else
			if(varCode & ctl3D) {						// make it look like a
				if(inColor) {							// scroll bar arrow
					if(!inactive) {						// with lighter center
						setPartColor(theControl, cTingeLight, true);
						GetForeColor(&grayColor);
						grayColor.red -= 13107;
						grayColor.green -= 13107;
						grayColor.blue -= 13107;
						RGBForeColor(&grayColor);
						PaintPoly((**hSpin).poly[inx]);	// filled color is light
						setPartColor(theControl, cTingeDark, true);
					}
				}
				else
				if(!(varCode & bigArrows))				// 3D, but in b&w, small
					PaintPoly((**hSpin).poly[inx]);		// arrow is solid fgColor
			}
			else										// not 3D
			if(!(varCode & bigArrows))					// so, small arrow
				PaintPoly((**hSpin).poly[inx]);			// is solid fgColor
				
			FramePoly((**hSpin).poly[inx]);				// frame the arrow
			
			if(!inactive) {								// set fg for next arrow
				if(inColor)
					setPartColor(theControl, cFrameColor, true);
				else
					ForeColor(blackColor);
			}
		}
	}
}

//==================================================================================
//	Get the rectangles for each part of the control.  These are used for
//	detecting mouse actions and drawing the control.
//==================================================================================

static void getRects	(ControlHandle theControl, short varCode, Rect parts[])
{
	short		aHt,aWid,fHt,cHt,wid,offset,inx;
	Rect 		base,work,rUp,rDown;
	FontInfo	f;
	
//----------------------------------------------------------------------------------
// set sizes for arrow height & width
//----------------------------------------------------------------------------------	
	if(varCode & bigArrows) {
		if(varCode & horizArrows) {
			aHt = ARROWID1;
			aWid = ARROWHT1;
		}
		else {
			aHt = ARROWHT1;
			aWid = ARROWID1;
		}
	}
	else {
		if(varCode & horizArrows) {
			aHt = ARROWID;
			aWid = ARROWHT;
		}
		else {
			aHt = ARROWHT;
			aWid = ARROWID;
		}
	}
	
//----------------------------------------------------------------------------------
// part rectangles for Arrow portions of control, right justified in control rect
//----------------------------------------------------------------------------------	
	base = (**theControl).contrlRect;					// the entire control	
	cHt = base.bottom-base.top;							// control height	
	parts[ALL] = parts[BASELINE] = base;	
		
	base.top = base.top + (cHt - aHt)/2;				// center arrow in rect
	base.bottom = base.top + aHt;
	
	rUp = work = base;				
	rUp.left = rUp.right - aWid;						// shift arrow to right		
	parts[FRAME] = rDown = rUp;							// rUp & rDown are full rects
	
	if(varCode & horizArrows)	
		rUp.left = rUp.right - (rUp.right - rUp.left)/2;// split them in half		
	else
		rUp.bottom = rUp.top + (rUp.bottom - rUp.top)/2;			
	parts[UPARROW] = rUp;								// rUp is top/right half
		
	if(varCode & horizArrows)	
		rDown.right = rUp.left-1;
	else	
		rDown.top = rUp.bottom+1;
	parts[DOWNARROW] = rDown;							// rDown is bottom/left half	
	
//----------------------------------------------------------------------------------
// set text drawing from font metrics
//----------------------------------------------------------------------------------	
	GetFontInfo(&f);
	fHt = f.ascent + f.descent+f.leading;
	if(f.leading == 0) 
		fHt++;
	
	offset = (aHt - fHt)/2;								// center digits in rect	
	work.bottom -= offset;
	work.right -= aWid;
	parts[TITLE] = work;
	parts[BASELINE].bottom = fHt - f.ascent;			// baseline for DrawStr	
}

//==================================================================================
//	Accelerate the control.  If we roll over a tens or hundreds FIVE times
//	and the mouse is still down, bump the increment. This lets the control
//	spin a number up by 1's from 0-50, by 10's from 50-100, then by 100's.
//==================================================================================

static void accelerate(ControlHandle theControl)
{
	long		mult;
	short		incr2;
	hSpinData	hSpin;
	
	hSpin = (hSpinData)(**theControl).contrlData;
	if(!hSpin)
		return;
		
	if((**hSpin).increment == 1)
		incr2 = 10;
	else
		incr2 = (**hSpin).increment*(**hSpin).increment;
		
	mult = (**theControl).contrlValue % incr2;
	if((**hSpin).currIncr == 
		(**hSpin).increment && !mult) {			// just rolled over a ten		
		if((**hSpin).bumpit == 1) {				// for the fifth time			
			(**hSpin).currIncr = incr2;			// bump our increment value		
			(**hSpin).bumpit = -3;
		}
		else {
			(**hSpin).bumpit += 1;				// for the first time			
		}
	}
	else
	if((**hSpin).currIncr == incr2 && !mult) {	// just rolled over a hundred	
		if((**hSpin).bumpit == 1) {				// for the fifth time			
			(**hSpin).currIncr = incr2*incr2;	// bump our increment value		
		}
		else {
			(**hSpin).bumpit += 1;				// for the first time			
		}
	}
}