//-------------------------  1991-1994 by James G. Stout --------------------------
// File		: cdefTog.c
// Date		: November 2,1991
// Author	: Jim Stout
// Purpose	: A CDEF to implement Tog's buttons
//			: use procID :   104 * 16 + varCode 
//			: when calling NewControl() or in your resource template.
//			: This CDEF supports the System 7 style gray drawing of an
//			: inactive control and the useWindFont varCode.
//			:
//			: If you find a use for this, I'd love to know about it.  Bug reports
//			: are always interesting.
//			:
//			: Internet    : JimS@WRQ.COM(work hours, PST)
//			: AppleLink   : WRQ			(daily)
//			: CompuServe  : 73240,2052	(weekly or so)
//			: AOL         : JasG		(weekly or so)
//			: eWorld      : Jim Stout	(weekly or so)
//----------------------------------------------------------------------------------

//#define _DEBUGCDEF 1

#include <Palettes.h>
#include <GestaltEqu.h>
#include <SetUpA4.h>
#include <Traps.h>
#include "cdefTog.h"

//----------------------------------------------------------------------------------
//	CDEF private data
//----------------------------------------------------------------------------------
typedef struct {
short	osVers;
short	txFont;
short	txSize;
}togData,**togHdl;

//----------------------------------------------------------------------------------
//  low-memory globals  - I know, but Apple's popUp CDEF 63 uses these
//----------------------------------------------------------------------------------

extern short SysFontFam : 0x0BA6;
extern short SysFontSize : 0x0BA8;

//----------------------------------------------------------------------------------
//	Function prototypes
//----------------------------------------------------------------------------------

void 		doDraw			(ControlHandle, short);
void		setPartColor	(ControlHandle, short);
void		saveColors		(RGBColor *);
void		restoreColors	(RGBColor *);
short		getPixDepth		(void);
short		getOSVers		(void);
Boolean		trapAvailable	(short);
short		numToolBoxTraps	(void);
TrapType	getTrapType		(short);

//----------------------------------------------------------------------------------
//	CDEF main for "Tog Button"
//----------------------------------------------------------------------------------
#ifdef _DEBUGCDEF
pascal long CDmain (short varCode, ControlHandle theControl, short message, long param);
pascal long CDmain (varCode, theControl, message, param)
#else
pascal long main (varCode, theControl, message, param)
#endif
short			varCode;
ControlHandle	theControl;
short			message;
long			param;
{
	short		txF,txS;
	long		ret = 0L;
	togHdl		hTog;
	Point		p;
	GrafPtr		thisPort;
		
	RememberA0();
	SetUpA4();
		
	if((**theControl).contrlData)
		HLock((**theControl).contrlData);
				
	GetPort(&thisPort);
	
// set window font & size info	
	
	txF = thisPort->txFont;
	txS = thisPort->txSize;
	if(!(varCode & useWindFont)) {				// use system font	
		thisPort->txFont = SysFontFam;
		thisPort->txSize = SysFontSize;
	}

// Process messages to control	

	if (message == initCntl) {
		hTog = (togHdl)NewHandle(sizeof(togData));
		(**hTog).osVers = getOSVers();
		(**hTog).txFont = txF;
		(**hTog).txSize = txS;
		(**theControl).contrlData = (Handle)hTog;
		HLock((**theControl).contrlData);
	}
	else
	if (message == dispCntl) {
		HUnlock((**theControl).contrlData);
		DisposeHandle((**theControl).contrlData);
		(**theControl).contrlData = 0;
	}
	if (message == drawCntl &&
		(**theControl).contrlVis != 0 &&
			((WindowPeek)(**theControl).contrlOwner)->visible) {
		doDraw(theControl, varCode);
	}
	else
	if (message == testCntl) {
		p.v=HiWord(param);
		p.h=LoWord(param);
		if(PtInRect(p,&(*theControl)->contrlRect)) 
			ret = 1L;
	}
	else
	if (message == calcCRgns)
			RectRgn((RgnHandle)(param & 0x7fffffffL), &(*theControl)->contrlRect);
	else
	if (message == calcCntlRgn || message == calcThumbRgn)
			RectRgn((RgnHandle)(param), &(*theControl)->contrlRect);

// restore window font & size info	

	if(!(varCode & useWindFont)) {
		thisPort->txFont = txF;
		thisPort->txSize = txS;
	}
	if((**theControl).contrlData)
		HUnlock((**theControl).contrlData);
	RestoreA4();
	return (ret);
}

//----------------------------------------------------------------------------------
//	The "Tog Button" CDEF drawing routine
//----------------------------------------------------------------------------------

void doDraw(ControlHandle theControl, short varCode)
{
	Rect		r;
	Point		p;
	RgnHandle	c;
	FontInfo	f;
	Boolean		grayOK=FALSE;
	PenState	penSt;
	short		pixDepth,osVers,vAdj,i,x,y;
	RGBColor	mBack,mFore,saveFore;
	GDHandle	hThisDevice;
	GrafPtr		thisPort;
	togHdl		hTog;
		
	hTog = (togHdl)(**theControl).contrlData;
	if(!hTog)
		return;
	GetPort(&thisPort);
	GetPenState(&penSt);
	osVers = (**hTog).osVers;
	pixDepth = getPixDepth();

// Draw the diamond shaped control 
	
	if(pixDepth > 1) {
		saveColors(&saveFore);							// save current ForeColor	
		setPartColor(theControl, cFrameColor);
	}	
	r = (*theControl)->contrlRect;						// get pen loc for drawing
	if(thisPort->txFont != (**hTog).txFont ||			// if font changed,
		thisPort->txSize != (**hTog).txSize) {			// clear rect to erase
		(**hTog).txFont = thisPort->txFont;				// old font
		(**hTog).txSize = thisPort->txSize;
		EraseRect(&r);
	}
	
	p.h = r.left+2;
	p.v = r.bottom - (r.bottom - r.top)/2 - 1;
	
	c = NewRgn();										// set new clipping region	
	GetClip(c);
	ClipRect(&r);
	
	if((*theControl)->contrlHilite != 1) {				// remove old hiliting		
		r.right = r.left+18;
		EraseRect(&r);
	}
		
	MoveTo(p.h,p.v);									// button outline			
	LineTo(p.h+7,p.v+7);
	LineTo(p.h+14,p.v);
	LineTo(p.h+7,p.v-7);
	LineTo(p.h,p.v);
	
	if((*theControl)->contrlHilite == 1) {
		MoveTo(p.h+1,p.v);								// button outline - hilited	
		LineTo(p.h+7,p.v+6);
		LineTo(p.h+13,p.v);
		LineTo(p.h+7,p.v-6);
		LineTo(p.h+1,p.v);
	}
	if((*theControl)->contrlValue != 0) {				// button center			
		x = p.h+4;
		y = p.v;
		for(i=1;i<=4;i++) {
			MoveTo(x,y);
			LineTo(x+3,y+3);
			x++;
			y--;
		}
		x = p.h+5;
		y = p.v;
		for(i=1;i<=3;i++) {
			MoveTo(x,y);
			LineTo(x+2,y+2);
			x++;
			y--;
		}
	}
	
// Draw the control title, in gray if it is inactive	

// Find out if we can draw in the new style gray		

	if(pixDepth > 1)	{								// are we in a color port ?		
		setPartColor(theControl, cTextColor);
		if((*theControl)->contrlHilite == 255 &&	 	// control is inactive			
				(**hTog).osVers >= 0x0700)	{			// and it's System 7			
			GetForeColor(&mFore);						// gray out control			
			GetBackColor(&mBack);						// the Sys 7 way			
			hThisDevice = GetGDevice();
			if(grayOK = GetGray(hThisDevice,&mBack,&mFore))
				RGBForeColor(&mFore);
		}
	}
	
// Draw the control title, in proper font	

	GetFontInfo(&f);									// draw the title 			
	vAdj= f.ascent + f.descent + f.leading;				// after positioning pen	
	p.v = r.bottom - ((r.bottom - r.top) - vAdj)/2;		// to a loc based on font	
	p.v -= (f.descent + f.leading);
	MoveTo(p.h+18,p.v);
	
	DrawString((*theControl)->contrlTitle);				// draw title of control	
	
	if(grayOK == FALSE &&								// gray out the old way		
		(*theControl)->contrlHilite == 255) {			// if control is inactive	
		PenPat( (ConstPatternParam) "\xAA\x55\xAA\x55\xAA\x55\xAA\x55");
		PenMode(patBic);
		r = (*theControl)->contrlRect;	
		r.left+=18;
		PaintRect(&r);	
	}
	
// Clean up and leave	

	SetClip(c);											// restore clip & pen		
	DisposeRgn(c);
	SetPenState(&penSt);
	if(pixDepth > 1)	
		restoreColors(&saveFore);						// restore ForeColor		
}
//----------------------------------------------------------------------------------
//	The following routines are miscellaneous support routines used by the Tog Button
//	CDEF.
//----------------------------------------------------------------------------------
void setPartColor(ControlHandle theControl, short part)
{
	short			inx,max;
	AuxCtlHandle	aux;
	CCTabHandle		ccTab;
	
	GetAuxCtl(theControl, &aux);
	ccTab = (**aux).acCTable;
	max = (**ccTab).ctSize;
	for(inx=0;inx<=max;inx++) {
		if((**ccTab).ctTable[inx].value == part) {
			RGBForeColor(&(**ccTab).ctTable[inx].rgb);
			break;
		}
	}
}

void saveColors(saveFore)
RGBColor	*saveFore;
{
	GetForeColor(saveFore);
}

void restoreColors(saveFore)
RGBColor	*saveFore;
{
	RGBForeColor(saveFore);
}

//----------------------------------------------------------------------------------
//	Get the pixel depth of the current GrafPort
//----------------------------------------------------------------------------------

short getPixDepth()
{
	OSErr		err;
	long		gResult;
	short		pixDepth = 1;							// assume a 1bit (B&W) port	
	GDHandle	hThisDevice;
	CGrafPtr	ourPort;
	
	if(trapAvailable(_Gestalt)) {						// is Gestalt available ?	
		err = Gestalt(gestaltQuickdrawVersion,&gResult);// check for QD version		
		if(err == noErr & LoWord(gResult > 0x0001)) {	// is it ColorQD ?			
			hThisDevice = GetGDevice();
			pixDepth = (**(**hThisDevice).gdPMap).pixelSize;
			GetPort((GrafPtr *)&ourPort);
			if(!(ourPort->portVersion & 0xC000))		// not a color port			
				pixDepth = 1;							// so it is 1 bit deep		
		}
	}
	return(pixDepth);
}

//----------------------------------------------------------------------------------
//	Use Gestalt to find out the MacOS version. 
//----------------------------------------------------------------------------------
short getOSVers()
{
	OSErr		err;
	short		ret=0x0600;
	long		gResult;
	
	if(trapAvailable(_Gestalt)) {						// is Gestalt available ?	
		err = Gestalt(gestaltSystemVersion,&gResult);
		if(err == noErr)
			ret = LoWord(gResult);
	}
	return(ret);
}

//----------------------------------------------------------------------------------
//	Generic routine to see if a given trap is available
//----------------------------------------------------------------------------------
Boolean	trapAvailable(theTrap)
short	theTrap;
{
	TrapType	tType;
	
	tType = getTrapType(theTrap);
	
	if(tType == ToolTrap) {
		theTrap &= 0x07ff;
		if(theTrap >= numToolBoxTraps())
			theTrap = _Unimplemented;
	}
	return(( NGetTrapAddress(theTrap, ToolTrap) !=
		NGetTrapAddress(_Unimplemented, ToolTrap) ));
}

//----------------------------------------------------------------------------------
//	Needed for "trapAvailable" routine
//----------------------------------------------------------------------------------
TrapType getTrapType(theTrap)
short	theTrap;
{
	if((theTrap &= 0x0800))
		return(ToolTrap);
	return(OSTrap);
}
//----------------------------------------------------------------------------------
//	Needed for "trapAvailable" routine
//----------------------------------------------------------------------------------
short numToolBoxTraps()
{
	if(NGetTrapAddress(_InitGraf, ToolTrap) ==
		NGetTrapAddress(0xaa6e, ToolTrap))
		return(0x0200);
	return(0x0400);
}
